#include <Rpc.h>

//* .dll file names
#define FILENAME_DLL_FD                    _T("FilesystemDialogs.dll")

//* Exported function names
#define NAME_FD_OpenDialog                 "FilesystemDialogs_OpenDialog"
#define NAME_FD_SaveDialog                 "FilesystemDialogs_SaveDialog"
#define NAME_FD_FolderDialog               "FilesystemDialogs_FolderDialog"
#define NAME_FD_Settings                   "FilesystemDialogs_SettingsWindow"
#define NAME_FD_InitTOpenDialogParams      "FilesystemDialogs_InitTOpenDialogParams"
#define NAME_FD_FreeTOpenDialogParams      "FilesystemDialogs_FreeTOpenDialogParams"
#define NAME_FD_InitTOpenDialogResults     "FilesystemDialogs_InitTOpenDialogResults"
#define NAME_FD_FreeTOpenDialogResults     "FilesystemDialogs_FreeTOpenDialogResults"

#define FD_MAX_PATH                     MAX_PATH * 30

//* Pointer to TOpenDialogResults is in wParam
//* Pointer to TOpenDialogParams is in lParam
//* You get the messages in the hookproc and to the specified Parent window handle
#define FDM_BASE                      	2 * WM_USER + 1

//* Filesystem Dialogs results
//* Pointer to TOpenDialogResults in wParam
//* You need to specify the ParentWindow also
#define FDM_RESULTS                   	FDM_BASE + 1

//* Load settings
#define FDM_LOADSETTINGS      			FDM_BASE + 2

//* Free memory that you have allocated for the dialog('s params, results)
#define FDM_FREEMEM           			FDM_BASE + 3

//* The dialog form is closing
#define FDM_CLOSING           			FDM_BASE + 4

//* The dialog form's window handle in wParam
//* Pointer to TOpenDialogParams is in lParam
#define FDM_WINDOW_HANDLE     			FDM_BASE + 5

//* Realing the integrated form to the parent
#define FDM_REALIGN_INTEGRATED_WINDOW 	FDM_BASE + 6

//* Sent on file name selection
//* Sent on file name selection
//* File name is in wParam as PChar (unicode), item's PIDL in lParam
#define FDM_ON_SELECT_FILE_NAME			FDM_BASE + 7

//* Refresh folder
#define FDM_REFRESH_FOLDER 				FDM_BASE + 8

//* Thumbnail is ready
#define FDM_THUMBNAIL_READY 			FDM_BASE + 9

//* Send thumbnail file name to thumbnail control thread
#define FDM_ADD_THUMBNAIL_FILENAME 		FDM_BASE+ 10

//* Thumbnailer thread finished
#define FDM_THUMBNAILER_THREAD_FINISHED FDM_BASE + 11

//* Terminate thumbnailer control thread
#define FDM_THUMBNAILER_CONTROL_THREAD_TERMINATE FDM_BASE + 12

//* Secondary viewer picture thread finished
#define FDM_SECONDARYVIEWER_PICTURE_READY FDM_BASE + 13

//* Secondary viewer audio picture thread finished
#define FDM_SECONDARYVIEWER_AUDIOPICTURE_READY FDM_BASE + 14

//* Viewer audio picture thread finished
#define FDM_VIEWER_AUDIOPICTURE_READY  	FDM_BASE + 15

//* Custom previewer query file (wParam = PChar file name, lParam = PIDL)
#define FDM_PREVIEWER_CUSTOM_QUERY  	FDM_BASE + 16
//* Custom previewer open previewer (Wnd = handle for the panel, wParam = PChar file name, lParam = PIDL)
#define FDM_PREVIEWER_CUSTOM_DISPLAY 	FDM_BASE + 17
//* Custom previewer close
#define FDM_PREVIEWER_CUSTOM_CLOSE  	FDM_BASE + 18
//* Custom previewer resized
#define FDM_PREVIEWER_CUSTOM_RESIZED 	FDM_BASE + 19

//* Viewer audio picture progress
#define FDM_VIEWER_AUDIOPICTURE_PROGRESS FDM_BASE + 20

//* Secondary viewer audio picture progress
#define FDM_SECONDARYVIEWER_VIEWER_AUDIOPICTURE_PROGRESS FDM_BASE + 21

//* Navigate to folder specified as PIDL in WPARAM
#define FDM_NAVIGATE_TO_PIDL = FDM_BASE + 22

//* OR these flags
enum TOpenOption
{
	ofReadOnly				= 1,
	ofOverwritePrompt		= 1 << 1,
	ofHideReadOnly			= 1 << 2,
	ofNoChangeDir			= 1 << 3,
	ofShowHelp				= 1 << 4,
	ofNoValidate			= 1 << 5,
	ofAllowMultiSelect		= 1 << 6,
	ofExtensionDifferent	= 1 << 7,
	ofPathMustExist			= 1 << 8,
	ofFileMustExist			= 1 << 9,
	ofCreatePrompt			= 1 << 10,
	ofShareAware			= 1 << 11,
	ofNoReadOnlyReturn		= 1 << 12,
	ofNoTestFileCreate		= 1 << 13,
	ofNoNetworkButton		= 1 << 14,
	ofNoDereferenceLinks	= 1 << 15,
	ofEnableIncludeNotify	= 1 << 16,
	ofDontAddToRecent		= 1 << 17,
	ofForceShowHidden		= 1 << 18,
	ofNotModal				= 1 << 19,
	ofNoClose				= 1 << 20,
	ofIntegrate				= 1 << 21,
	ofStayOnTop				= 1 << 22,
	ofNoCloseButton		  	= 1 << 23,
	ofOnSelectFilename		= 1 << 24,
	ofCompatibilityMode		= 1 << 25,
	ofCustomFileInfo		= 1 << 26,
	ofCustomThumbnail		= 1 << 27,
	ofCustomPreViewer		= 1 << 28,
	ofInitialFolderIsPIDL   = 1 << 29
}; // TOpenOption

//* OR these flags
enum TOpenOptionEx
{
	ofExNoPlacesBar				= 1, 		// Not used
	ofExNoStatusBar				= 1 << 1,
	ofExNoDrivesFavsRecents		= 1 << 2,
	ofExNoFileName				= 1 << 3,
	ofExNoFilter				= 1 << 4,
	ofExNoToolBar				= 1 << 5,
	ofExNoAddToFavs				= 1 << 6,
	ofExNoTreeViewAndToolBar 	= 1 << 7,
	ofExNoPathAndFilter			= 1 << 8
}; // TOpenOptionEx

//* from CommDlg.pas
typedef UINT(__stdcall *OFNHookProc)(HWND Wnd, UINT Msg, WPARAM wParam, LPARAM lParam);  //* stdcall

typedef BOOL(__stdcall *FDFileInfoHookProc)(LPWSTR FileName, void* PIDL, LPCTSTR& FileInformation, void* User);
typedef BOOL(__stdcall *FDFileThumbnailHookProc)(LPWSTR FileName, void* PIDL, int Width, int Height, HBitmap& Thumbnail,; __int64& OriginalWidth, __int64& OriginalHeight, void* User);


#define ofExNoPlacesBar 1

//* Filesystem dialogs types
enum TFSDialogType
{
    fsdtOpenDialog = 0,
    fsdtSaveDialog = 1,
    fsdtFolderDialog = 2
}; // TFSDialogType

//* Filesystem dialog Open file/Save file/Folder dialog Params
typedef struct {
	LPWSTR DefaultExt;          					//* Default extension
	LPWSTR FileName;            					//* Default file name for Open/Save and Initial dir
	LPWSTR Filters;              					//* Filters eg.: 'Image files (*.bmp,*.jpg)|*.bmp;*.jpg|All files|*.*'
	int FilterIndex;           						//* Filter to use default, 1-first, 2-second...
	int HelpContext;           						//* Help context (not yet implemented)
	LPWSTR InitialDir;          					//* Initial folder, this is extracted first from FileName, if no FileName specified settings are used to decide which folder to use, if 'ofInitialFolderIsPIDL' is set this should be a pointer to ItemIDList (a PItemIDList)
	int Options;       								//* TOpenOptions: Dialog options (not all implemented so far), OR TOpenOption flags together
	int OptionsEx;             						//* Extra Dialog options, specify panels to hide (not all implemented), OR TOpenOptionEx flags together
	LPWSTR Title;               					//* Title to display as window's caption
	HWND ParentWindow;         						//* Parent window handle (needed for HookProc)
	GUID DialogGUID;          						//* Specifically identificate the dialog (you can get the same dialog always with this flag)
	OFNHookProc HookProc;      						//* Hookproc (for not modal mode, ParentWindow handle is also needed)
	int DialogType;  								//* Type of the dialog: Open/Save/Folder
	boolean AllowTIFFLZW;     						//* Allow LZW coded TIF image thumbnails (using these formats requires a license)
	//FileSize: Int64;								//* Estimated file size (will warn if won't fit in the selected directory before performing any saving)
	//MD5: String;									//* MD5 value of the file to save (compare to exisisting files)
	//User: Pointer;            					//* User variable
	FDFileInfoHookProc FileInfoHookProc;			//* Callback that gives back custom file info
	LPWSTR FileInfoColumnName;  					//* Column name used for custom file info
	FDFileThumbnailHookProc FileThumbnailHookProc; 	//* Callback that gives back thumbnail
} *POpenDialogParams, TOpenDialogParams;

//* Filesystem dialog result
typedef struct {
    BOOL Successfull;          	//* True: User pressed the Open/Save, False: User pressed Cancel, closed the window or error (there is a fallback to comdlg32.dll though)
	LPWSTR FileName;           	//* The filename the user selected (only one) ending with a double null
    long long FileNameSize;    	//* FileName size in bytes (should be always at least MAX_PATH)
    UINT FileCount;           	//* Number of files, if user selected multiple files then FileCount > 1
    LPWSTR FileNames;           //* List of files in format "C:\Dir\Data.txt" separated by null characters ending with a double null
    long long FileNamesSize;    //* FileNames size in bytes (it's up to you to validate last entry if all selected filenames don't fit into the FileNames)
    int FilterIndex;           	//* Filter's index the user selected 1-first, 2-second, etc...
}*POpenDialogResults, TOpenDialogResults;

//* FDM_ON_SELECT_FILE_NAME in WPARAM
typedef struct {
	LPWSTR FileName;            //* The filename the user selected (only one)
    void* PIDL;        			//* File's item id list (PIDL)
}*POpenDialogOnSelectFileName, TOpenDialogOnSelectFileName;

//* Exported functions
typedef BOOL(__stdcall *t_OpenDialog)(TOpenDialogParams Params, POpenDialogResults Results);
typedef BOOL(__stdcall *t_SaveDialog)(TOpenDialogParams Params, POpenDialogResults Results);
typedef BOOL(__stdcall *t_FolderDialog)(TOpenDialogParams Params, POpenDialogResults Results);
typedef BOOL(__stdcall *t_FilesystemDialogsSettings)();

typedef bool(__stdcall *t_InitTOpenDialogParams)(POpenDialogParams Struct, long long MaxFiles/* = 1000*/);
typedef bool(__stdcall *t_FreeTOpenDialogParams)(POpenDialogParams Struct);
typedef bool(__stdcall *t_InitTOpenDialogResults)(POpenDialogResults Res, long long MaxFiles/* = 1000*/);
typedef bool(__stdcall *t_FreeTOpenDialogResults)(POpenDialogResults Struct);

namespace FileSystemDialogs
{
	bool InitFileSystemDialogsLibrary();
	bool FreeFileSystemDialogsLibrary();

	extern t_OpenDialog FilesystemDialogs_OpenDialog;
	extern t_SaveDialog FilesystemDialogs_SaveDialog;
    extern t_FolderDialog FilesystemDialogs_FolderDialog;
    extern t_FilesystemDialogsSettings FilesystemDialogs_Settings;
    extern t_InitTOpenDialogParams FilesystemDialogs_InitTOpenDialogParams;
    extern t_FreeTOpenDialogParams FilesystemDialogs_FreeTOpenDialogParams;
    extern t_InitTOpenDialogResults FilesystemDialogs_InitTOpenDialogResults;
    extern t_FreeTOpenDialogResults FilesystemDialogs_FreeTOpenDialogResults;
}