unit DPIFunctions;

interface

Uses
    Windows,
    Winapi.MultiMon,
    VCL.Forms;

//type
    //HMONITOR = THandle;

Type
  TMONITOR_DPI_TYPE = (
    MDT_EFFECTIVE_DPI {= 0},
    MDT_ANGULAR_DPI   {= 1},
    MDT_RAW_DPI       {= 2},
    MDT_DEFAULT       {= MDT_EFFECTIVE_DPI });

    function GetSystemDPI(monitor: HMONITOR): Integer;

    procedure GetMonitorDPIWin8(monitor: HMONITOR; dpiType: TMONITOR_DPI_TYPE; var dpiX, dpiY: UINT);
    procedure GetDesktopDPI(var dpiX, dpiY: UINT);

var
    GetDpiForMonitor: function(monitor: HMONITOR; dpiType: TMONITOR_DPI_TYPE; var dpiX, dpiY: UINT): HRESULT; stdcall;

implementation

function GetSystemDPI(monitor: HMONITOR): Integer;
var
    dpiX, dpiY: UINT;
begin
    //Result := 96;
    //* Get DPI (from Windows 8)
    GetMonitorDPIWin8(monitor, MDT_EFFECTIVE_DPI, dpiX, dpiY);
    if dpiX = 0 then begin
        //* Pre Windows 8
        GetDesktopDPI(dpiX, dpiY);
        if dpiX = 0 then begin
            Result := Screen.PixelsPerInch;
        end else begin
            if dpiX > dpiY then begin
                Result := dpiY;
            end else begin
                Result := dpiX;
            end;
        end;
    end else begin
        if dpiX > dpiY then begin
            Result := dpiY;
        end else begin
            Result := dpiX;
        end;
    end;
end;

procedure GetMonitorDPIWin8(monitor: HMONITOR; dpiType: TMONITOR_DPI_TYPE; var dpiX, dpiY: UINT);
var
    //ErrCode: HResult;
    hShcore: THandle;
begin
    dpiX := 0;
    dpiY := 0;
    hShcore := GetModuleHandle('Shcore');
    If hShcore <> 0 then begin
        GetDpiForMonitor := GetProcAddress(hShcore,'GetDpiForMonitor');
    end;
    If @GetDpiForMonitor <> nil then begin
        {ErrCode :=} GetDpiForMonitor(Monitor, MDT_EFFECTIVE_DPI, dpiX, dpiY);
    end;
end;

/// <summary>Determines the current screen resolution in DPI.</summary>
/// <returns>Point.X is the X DPI, Point.Y is the Y DPI.</returns>
procedure GetDesktopDPI(var dpiX, dpiY: UINT);
var
    DC: hDC;
begin
    DC := GetDC(0);

    dpiX := GetDeviceCaps(DC, LOGPIXELSX);
    dpiY := GetDeviceCaps(DC, LOGPIXELSY);

    ReleaseDC(0, DC);
end;

end.
