//********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs Library 2.1.3.6840  3delite 2006-2024                                                                *
//*     See Filesystem Dialogs Library ReadMe.txt for details.                                                                   *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* 	http://www.shareit.com/product.html?productid=300958160                                                                  *
//* Shareware License: 95                                                                                                       *
//* 	http://www.shareit.com/product.html?productid=300070165                                                                  *
//* Commercial License: 450                                                                                                     *
//* 	http://www.shareit.com/product.html?productid=300070166                                                                  *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                          *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit FilesystemDialogsDefs;

{$IF CompilerVersion >= 24}
    {$LEGACYIFEND ON}
{$IFEND}

interface

Uses
    SysUtils, UITypes, Classes, Windows, Dialogs, Forms, Graphics, ShlObj;

const
    //* .dll file names
    FILENAME_DLL_FD                       = 'FilesystemDialogs.dll';

    //* Exported function names
    NAME_FD_OpenDialog                    = 'FilesystemDialogs_OpenDialog';
    NAME_FD_SaveDialog                    = 'FilesystemDialogs_SaveDialog';
    NAME_FD_FolderDialog                  = 'FilesystemDialogs_FolderDialog';
    NAME_FD_Settings                      = 'FilesystemDialogs_SettingsWindow';
    NAME_FD_FreeDialogResults             = 'FilesystemDialogs_FreeDialogResults';

    //* Registry keys
    STR_REG_FD_SETTINGS                   = 'Software\3delite\Filesystem Dialogs';
    STR_REG_FD_FONT_SETTINGS              = 'Font settings';
    STR_REG_FD_DIALOG_SETTINGS            = 'App settings';
    STR_REG_FD_DIALOG_LASTCONFIG          = 'Last config';
    STR_REG_FD_FAVORITE_FOLDERS           = 'Favourite folders';
    STR_REG_FD_LAST_FOLDERS               = 'Last visited folders';
    STR_REG_FD_FAVORITE_FOLDERS_PIDL      = 'Favourite folders PIDL';
    STR_REG_FD_LAST_FOLDERS_PIDL          = 'Last visited folders PIDL';
    STR_REG_FD_PROCESS_SETTINGS           = 'Process settings';
    STR_REG_FD_LISTER_LAYOUTS             = 'Lister layouts';
    STR_REG_FD_LISTER_LAYOUT_LAST         = 'Lister layout last';

    //* Global settings
    STR_REG_USE_LAST_FOLDER               = 'Always use last used folder';
    STR_REG_SINGLE_SETUP                  = 'Single setup';
    STR_REG_FILE_DATETIME_PRECISION       = 'File date time ms precision';
    STR_REG_READ_CREATED_FOLDER           = 'Automatically go into newly created folder';
    STR_REG_SAVE_SETTINGS_ON_CANCEL       = 'Save settings on Cancel';
    STR_REG_THUMBNAILS_FOR_DIRS           = 'Thumbnails for dirs';
    STR_REG_THUMBNAILS_FOR_DIR_AUDIO      = 'Thumbnails for dirs with audio';
    STR_REG_THUMBNAILS_FOR_AUDIOFILES     = 'Thumbnails for recognised audio files';
    STR_REG_THUMBNAILS_FOR_AUDIOFILES_SHADOW = 'Thumbnails for recognised audio files shadow';
    STR_REG_THUMBNAILS_FOR_TEXTFILES      = 'Thumbnails for TEXT files';
    STR_REG_THUMBNAILS_GIF_ANIM           = 'Thumbnails play animated GIFs';
    STR_REG_THUMBNAILS_BCKG_COLOR         = 'Thumbnails use custom color for non image thumbnail background';
    STR_REG_THUMBNAILS_BCKG_COLOR_V       = 'Thumbnails custom color for non image thumbnail background';
    STR_REG_THUMBNAILS_BCKG_PIC_COLOR     = 'Thumbnails use custom color for picture thumbnail background';
    STR_REG_THUMBNAILS_BCKG_PIC_COLOR_V   = 'Thumbnails custom color for picture thumbnail background';
    STR_REG_THUMBNAILS_ZOOM_X2            = 'Thumbnails zoom small images x2';
    STR_REG_THREAD_THUMBNAILER            = 'Thumbnailer threaded';
    STR_REG_THUMBNAILS_WIDTH              = 'Thumbnails width';
    STR_REG_THUMBNAILS_HEIGHT             = 'Thumbnails height';
    STR_REG_THUMBNAILS_BALLOON_WIDTH      = 'Thumbnails balloon width 2';
    STR_REG_THUMBNAILS_BALLOON_HEIGHT     = 'Thumbnails balloon height 2';
    STR_REG_THUMBNAILS_SKEEZ_WIDTH        = 'Thumbnails Skeez width';
    STR_REG_THUMBNAILS_SKEEZ_HEIGHT       = 'Thumbnails Skeez height';
    STR_REG_THUMBNAILS_SIZE_PRESET        = 'Thumbnails dimensions preset';
    STR_REG_THUMBNAILS_SAMPLE_COLOR_L     = 'Thumbnails audio sample color left channel';
    STR_REG_THUMBNAILS_SAMPLE_COLOR_R     = 'Thumbnails audio sample color right channel';
    STR_REG_THUMBNAILS_SAMPLE_COLOR_B     = 'Thumbnails audio sample color background';
    STR_REG_THUMBNAILS_SAMPLE_COLOR_L_VIEWER = 'Viewer audio sample color left channel';
    STR_REG_THUMBNAILS_SAMPLE_COLOR_R_VIEWER = 'Viewer audio sample color right channel';
    STR_REG_THUMBNAILS_SAMPLE_COLOR_B_VIEWER = 'Viewer audio sample color background';
    STR_REG_THUMBNAILS_SAMPLE_AA          = 'Thumbnails audio sample antialiasing level';
    STR_REG_THUMBNAILS_FOLDER_COLOR       = 'Thumbnails folder icon color';
    STR_REG_THUMBNAILS_ALL_THREADS        = 'Thumbnails use all threads';
    STR_REG_SAVE_SESC                     = 'Save SESC';
    STR_REG_FS_FORCE_EXTCASE              = 'Force extension case';
    STR_REG_FS_FORCE_EXTCASEL             = 'Force extension to uppercase';
    STR_REG_FS_FORCE_EXTCASEU             = 'Force extension to lowercase';
    STR_REG_INACTIVE_TREEVIEW             = 'Inactive tree view until first touch';
    STR_REG_OMIT_FILE_EXT                 = 'Omit file extensions';
    STR_REG_DETAILS                       = 'Display details in detailed list mode';
    STR_REG_SETTINGS_TOP                  = 'Settings window top';
    STR_REG_SETTINGS_LEFT                 = 'Settings window left';
    STR_REG_SETTINGS_WIDTH                = 'Settings window width';
    STR_REG_SETTINGS_HEIGHT               = 'Settings window height';
    STR_REG_SETTINGS_PAGE_INDEX           = 'Settings page index';
    STR_REG_SETTINGS_PAGE_THUMBS_INDEX    = 'Settings thumbs page index';
    STR_REG_SETTINGS_PAGE_PREFS_INDEX     = 'Settings preferences page index';
    STR_REG_RECENT_FOLDER_COUNT           = 'Number of recent folders to remember';
    STR_REG_SHOW_SCROLLBAR_RECENT         = 'Show Scrollbars with Recent Folder and Drives and Favorites list';
    STR_REG_SHOW_SCROLLBAR_TREEVIEW       = 'Show Scrollbars on Tree View';
    STR_REG_THUMBNAILS_CACHED_GRAPHICAL   = 'Cache the graphical thumbnails in PNG database';
    STR_REG_THUMBNAILS_CACHED_AUDIO       = 'Cache the audio thumbnails in database';
    STR_REG_SETTINGS_DISPLAY_EXTS_CONVERTED = 'Display file extensions converted';
    STR_REG_SEC_VIEWER                    = 'Enable secondary display full screen viewer';
    STR_REG_SEC_VIEWER_SPECIFY            = 'Secondary display full screen viewer specify display';
    STR_REG_SEC_VIEWER_MONITOR            = 'Secondary display full screen viewer monitor number';
    STR_REG_SEC_VIEWER_ONHOVER            = 'Secondary display full screen viewer on hover';
    STR_REG_SEC_VIEWER_AUDIO              = 'Secondary display full screen audio viewer';
    STR_REG_SEC_VIEWER_AUDIO_NO_STATUS    = 'Secondary display full screen audio viewer don''t show progress';
    STR_REG_SEC_VIEWER_AUDIO_NO_INFO      = 'Secondary display full screen audio viewer don''t show audio information';
    STR_REG_SEC_VIEWER_AUDIO_AA           = 'Secondary display full screen audio viewer antialiasing level';
    STR_REG_SEC_VIEWER_PICTURE            = 'Secondary display full screen picture viewer';
    STR_REG_SEC_VIEWER_PICTURE_RESAMPLER  = 'Secondary display full screen picture viewer resampler filter';
    STR_REG_SEC_VIEWER_HTML               = 'Secondary display full screen HTML viewer';
    STR_REG_SEC_VIEWER_TEXT_RTF           = 'Secondary display full screen TEXT & RTF viewer';
    STR_REG_BALLOON_ON_OFF_GLOBAL         = 'Balloon hints enabled (global)';
    STR_REG_BALLOON_IN_LISTER             = 'Show balloon hints in Lister mode';
    STR_REG_BALLOON_IN_THUMBNAIL          = 'Show balloon hints in Thumbnail mode';
    STR_REG_BALLOON_HINT_TIMEOUT          = 'Balloon hint time-out';
    STR_REG_BALLOON_THUMBNAILS            = 'Display thumbnails on balloon hints';
    STR_REG_BASS_DONTINIT                 = 'Do not initialize BASS';
    STR_REG_TYPES_BROWSER                 = 'Web browser file extension types';
    STR_REG_TYPES_AUDIO                   = 'Audio file extension types';
    STR_REG_TYPES_TEXT                    = 'Text file extension types';
    STR_REG_TYPES_VIDEO                   = 'Video file extension types';
    STR_REG_BALLOON_IN_LOCATIONS          = 'Show balloon hints on Drives & Favs and Recent Folder list';
    STR_REG_URL_DECODE_FILENAMES          = 'Automatically decode URL encoded file names';
    STR_REG_UPPERCASE_FIRST_LETTER_FOR_FILENAMES = 'Automatically convert first letter in file names to upper case';
    STR_REG_UNDERSCORES_2_SPACE_FOR_FILENAMES = 'Automatically convert underscores to spaces in file names';
    STR_REG_RECENT_FILES_COUNT            = 'Number of recent files to remember';
    STR_REG_DONTDISAPLAY_TFDDB            = 'Do not show "Thumbs Filesystem Dialogs.db" file';
    STR_REG_SHOW_HIDDENSYSTEM             = 'Show hidden/system files';
    STR_REG_FILENAME_TEMPLATE             = 'Filename template';
    STR_REG_RECENT_FILES_DEL_INVALID      = 'Automatically delete invalid recent link files';
    STR_REG_THUMBNAILS_FOR_AUDIOFILES_APIC = 'Thumbnails for recognised audio files from APIC';
    STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_APIC = 'Thumbnails audio prefer cover arts';
    STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_SAMPLES = 'Thumbnails audio prefer samples';
    STR_REG_THUMBNAILER_TEXT_ZOOM         = 'Zoom percent value for text thumbnails font size';
    STR_REG_SEC_VIEWER_HTML_UNKNOWN_TYPES = 'Secondary display full screen HTML viewer display unknown types too';
    STR_REG_FILENAME_AUTOMATIC_RENAME     = 'Automatically rename existing save file names';
    STR_REG_NO_CDN_TYPECHANGE             = 'Do not send CDN_TYPECHANGE messages';
    STR_REG_TRY_WEBBROWSER                = 'Try previewing unknown file types in integrated web browser';
    STR_REG_LISTER_HAS_THE_FOCUS          = 'After window had appeared the Lister has the input focus';
    STR_REG_BALLOON_NO_FILENAME_FOCUS     = 'Do not show Balloon Hints when file name combo box has focus';
    STR_REG_LISTER_USE_CUSTOM_FONT_COLOR  = 'Lister use custom font color';
    STR_REG_LISTER_CUSTOM_FONT_COLOR      = 'Lister custom font color';
    STR_REG_LISTER_USE_CUSTOM_BACKGROUND_COLOR= 'Lister use custom background color';
    STR_REG_LISTER_CUSTOM_BACKGROUND_COLOR= 'Lister custom background color';
    STR_REG_DONT_ASK_IF_NO_FILE_TYPE_SPECIFIED = 'Do not ask if no file type specified';
    STR_REG_THUMBNAILS_FOLDER_WIN_ICON    = 'Thumbnails use Windows folder icon';
    STR_REG_UI_FONT_SIZE                  = 'Dialog UI font size';
    STR_REG_UI_STYLE                      = 'UI style';
    STR_REG_UI_STYLE_WINDOW_OVERRIDE      = 'UI style override window';
    STR_REG_UI_STYLE_INTEGRATED_OVERRIDE  = 'UI style override integrated';
    STR_REG_COLUMN_LAYOUT_USED_NAME       = 'Used column layout name';
    STR_REG_TOGGLE_SELECTION_MODE_DETAILS = 'Toggle selection mode in lister';
    STR_REG_ANIMATED_MODE_ANIMATE         = 'Animated mode in animated lister';

//* Default file types
const
    FILE_TYPES_DEF_FILTER = 'Text files|*.txt;*.log;*.ini;*.cfg;*.text'
        + 'Picture files|*.jpg;*.bmp;*.png;*.gif;*.jpeg;*.pcx;*.psd;*.iff'
        + 'Audio files|*.mp3;*.mp2;*.mp1;*.flac;*.fla;*.flc;*.oga;*.wav;*.aif;*.aiff;*.aifc;*.afc;*.mp4;*.aac;*.ac3;*.mpc;*.mpa;*.wma;*.dff;*.dsf'
        + 'Document files|*.doc;*.rtf;*.txt;*.log'
        + 'Text system files|*.ini;*.cfg;*.bat;*.cmd;*.log'
        + 'All files|*.*';

    //* Viewer default file types
    FILE_TYPES_BROWSER                    = 'HTM, HTML, MHT, SWF, URL, XML, PDF';
    FILE_TYPES_AUDIO                      = 'WAV, AIF, AIFF, AIFC, AFC, MP3, MP2, MP1, MPA, M4A, M4B, AAC, ALAC, OGG, MOD, MO3, IT, XM, S3M, MTM, UMX, FLAC, FLA, FLC, OGA, APE, OFR, WV, MPC, WMA, DFF, DSF, AC3, OPUS, TTA';
    FILE_TYPES_TEXT                       = 'TEXT, TXT, CFG, INI, LOG, PAS, CPP, H, BAS';
    FILE_TYPES_VIDEO                      = 'AVI, MPG, MPEG, DIVX, XVID, TS, MKV, WMV, ASF, MP4';

    //* Filenames
    {$IFDEF DEBUG}
    FILENAME_DEBUG                        = 'E:\Filesystem Dialogs debug.log';
    {$ENDIF}

    WWW_FILESYSTEMDIALOGS_HELP 		      = 'https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/filesystemdialogs.html';

    //* Internal max. path size
    FD_MAX_PATH                           = MAX_PATH * 10;

    FILENAME_SEARCH_FILE_NAME_VALUE = 'Search file name values.txt';
    FILENAME_SEARCH_VALUE           = 'Search string values.txt';
    FILENAME_SEARCH_SETTINGS        = 'Search settings.ini';

    STR_MSG_FDM_UPDATE_STYLE = 'Filesystem Dialogs Update Style';

const
    cColName            = 0;
    cColSize            = 1;
    cColType            = 2;
    cColDetails         = 3;
    cColCreated         = 4;
    cColModified        = 5;
    cColAccessed        = 6;
    //* Audio tags
    cColArtist          = 7;
    cColTitle           = 8;
    cColAlbum           = 9;
    cColRating          = 10;
    cColTrack           = 11;
    cColGenre           = 12;
    cColYear            = 13;
    cColPlayTime        = 14;
    //* EXIF tags
    cColAuthor          = 15;
    cColPictureTitle    = 16;
    cColDateTaken       = 17;
    //* Video
    cColVideoTitle      = 18;
    cColDimensions      = 19;
    cColCodec           = 20;
    cColFrameRate       = 21;
    cColAudioSampleRate = 22;
    cColAudioChannels   = 23;
    cColAudioCodec      = 24;
    cColVideoPlayTime   = 25;
    //* Drive
    cColDriveFreeSpace  = 26;
    cColDriveCapacity   = 27;
    cColDriveGauge      = 28;
    //* Misc
    cColPath            = 29;

const
    STR_THUMB_Sampling                = ' Sampling... ';
    STR_THUMB_Generating              = ' Generating...';
    STR_THUMB_SESC_Saving             = 'Writing cache...';

    HEADER_COLUMN_NAME                = 'Name';
    HEADER_COLUMN_SIZE                = 'Size';
    HEADER_COLUMN_DESCRIPTION         = 'Description';
    HEADER_COLUMN_DETAILS             = 'Details';
    HEADER_COLUMN_CREATION_TIME       = 'Creation Time';
    HEADER_COLUMN_LAST_WRITE_TIME     = 'Modified Time';
    HEADER_COLUMN_LAST_ACCES_TIME     = 'Last Access Time';

const
    //* Blue for XP's white background
    THUMBNAIL_COLOR_L = $FF0000FF;
    THUMBNAIL_COLOR_R = $FF000064;

    //* Abs(0) = none
    THUMBNAIL_COLOR_B = $00000000;

const
    THUMBNAIL_COLOR_1L = $FFFF8000;
    THUMBNAIL_COLOR_1R = $FFFFFF00;
    THUMBNAIL_COLOR_1B = $00000000;

    THUMBNAIL_COLOR_2L = $FFFFFF00;
    THUMBNAIL_COLOR_2R = $FF00FF00;
    THUMBNAIL_COLOR_2B = $00000000;

    THUMBNAIL_COLOR_3L = $FF0000FF;
    THUMBNAIL_COLOR_3R = $FF00FF00;
    THUMBNAIL_COLOR_3B = $00000000;

    THUMBNAIL_COLOR_4L = $FFFF8000;
    THUMBNAIL_COLOR_4R = $FF00FF00;
    THUMBNAIL_COLOR_4B = $00000000;

type
    TColor = -$7FFFFFFF-1..$7FFFFFFF;
    TColor32 = type Cardinal;

//* Many of the following flags are not supported (legacy Windows flags), if any of the missing flags are needed, please contact me.
type
    TOpenOption = (ofReadOnly, ofOverwritePrompt, ofHideReadOnly, ofNoChangeDir, ofShowHelp, ofNoValidate, ofAllowMultiSelect, ofExtensionDifferent, ofPathMustExist, ofFileMustExist, ofCreatePrompt, ofShareAware,
        ofNoReadOnlyReturn, ofNoTestFileCreate, ofNoNetworkButton, ofNoDereferenceLinks, ofEnableIncludeNotify, ofDontAddToRecent, ofForceShowHidden,
        ofNotModal,             //* Not modal dialogs - user can access the app.'s UI
        ofNoClose,              //* Dialog will not close after double clicking or clicking on 'Open/Save' button - usefull with the 'ofNotModal' flag
        ofIntegrate,            //* The dialog will appear in the control specified by 'TOpenDialogParams.ParentWindow' - use with 'ofNotModal' and 'ofNoClose' flag
        ofStayOnTop,            //* The dialog will stay on top of other windows
        ofNoCloseButton,        //* Not possible to close the dialog if 'ofIntegrate' is used - usefull with the 'ofNotModal' flag
        ofOnSelectFilename,     //* When user selects a file (clicks on it) the hookproc will be called and/or message will be sent to the 'ParentHandle' - usefull with the 'ofNotModal' flag
        ofCompatibilityMode,
        ofCustomFileInfo,       //* Custom file info is implemented, 'TOpenDialogParams.FileInfoHookProc' must be assigned
        ofCustomThumbnail,      //* Custom thumbnail is implemented, 'TOpenDialogParams.FileThumbnailHookProc' must be assigned
        ofCustomPreViewer,      //* Custom viewer is implemented, hookproc must be assigned
        ofInitialFolderIsPIDL,  //* 'TOpenDialogParams.InitialDir' is not a pointer to a string but is a PIDL
        ofReturnPIDLs,          //* Only works with open and folder dialog, also file(s) must exist
        ofBrowseArchives        //* Only works with open dialog and with 'ofReturnPIDLs'
    );

    TOpenOptions = set of TOpenOption;

type
    TOpenOptionEx = (ofExNoPlacesBar, ofExNoStatusBar, ofExNoDrivesFavsRecents, ofExNoFileName, ofExNoFilter, ofExNoToolBar, ofExNoAddToFavs, ofExNoTreeViewAndToolBar, ofExNoPathAndFilter, ofExNoTextButtons, ofExNoOpenButtons, ofExShellOpen);

    TOpenOptionsEx = set of TOpenOptionEx;

type
    OFNHookProc = function(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT; stdcall;

type
    FDFileInfoHookProc = function(FileName: PWideChar; PIDL: PItemIDList; var FileInformation: PWideChar; User: Pointer): Bool; stdcall;
    FDFileThumbnailHookProc = function(FileName: PWideChar; PIDL: Pointer; Width: Integer; Height: Integer; var Thumbnail: HBitmap; var OriginalWidth: Int64; var OriginalHeight: Int64; User: Pointer): Bool; stdcall;

type
    tagOFN = {$IFDEF WIN32}packed{$ENDIF} record
        lStructSize: DWORD;
        hWndOwner: HWND;
        hInstance: HINST;
        lpstrFilter: PAnsiChar;
        lpstrCustomFilter: PAnsiChar;
        nMaxCustFilter: DWORD;
        nFilterIndex: DWORD;
        lpstrFile: PAnsiChar;
        nMaxFile: DWORD;
        lpstrFileTitle: PAnsiChar;
        nMaxFileTitle: DWORD;
        lpstrInitialDir: PAnsiChar;
        lpstrTitle: PAnsiChar;
        Flags: DWORD;
        nFileOffset: Word;
        nFileExtension: Word;
        lpstrDefExt: PAnsiChar;
        lCustData: LPARAM;
        lpfnHook: OFNHookProc;
        lpTemplateName: PAnsiChar;
        pvReserved: Pointer;
        dwReserved: DWORD;
        FlagsEx: DWORD;
    end;

type
    tagOFNW = {$IFDEF WIN32}packed{$ENDIF} record
        lStructSize: DWORD;
        hWndOwner: HWND;
        hInstance: HINST;
        lpstrFilter: PWideChar;
        lpstrCustomFilter: PWideChar;
        nMaxCustFilter: DWORD;
        nFilterIndex: DWORD;
        lpstrFile: PWideChar;
        nMaxFile: DWORD;
        lpstrFileTitle: PWideChar;
        nMaxFileTitle: DWORD;
        lpstrInitialDir: PWideChar;
        lpstrTitle: PWideChar;
        Flags: DWORD;
        nFileOffset: Word;
        nFileExtension: Word;
        lpstrDefExt: PWideChar;
        lCustData: LPARAM;
        lpfnHook: OFNHookProc;
        lpTemplateName: PWideChar;
        pvReserved: Pointer;
        dwReserved: DWORD;
        FlagsEx: DWORD;
    end;

type
    TCallData = record
        CalledFromComdlg32: Boolean;
        CalledUnicode: Boolean;
        UseBackupedStruct: Boolean;
    end;

type
    TSHBrowseinfoA = record
        hwndOwner: HWND;
        pidlRoot: PItemIDList;
        pszDisplayName: PAnsiChar;  { Return display name of item selected. }
        lpszTitle: PAnsiChar;      { text to go in the banner over the tree. }
        ulFlags: UINT;           { Flags that control the return stuff }
        lpfn: TFNBFFCallBack;
        lParam: LPARAM;          { extra info that's passed back in callbacks }
        iImage: Integer;         { output var: where to return the Image index. }
    end;

type
    TSHBrowseinfoW = record
        hwndOwner: HWND;
        pidlRoot: PItemIDList;
        pszDisplayName: PWideChar;  { Return display name of item selected. }
        lpszTitle: PWideChar;      { text to go in the banner over the tree. }
        ulFlags: UINT;           { Flags that control the return stuff }
        lpfn: TFNBFFCallBack;
        lParam: LPARAM;          { extra info that's passed back in callbacks }
        iImage: Integer;         { output var: where to return the Image index. }
    end;

Const
    WM_USER                 = $0400;
    OFN_OVERWRITEPROMPT     = $00000002;
    OFN_ALLOWMULTISELECT    = $00000200;
    OFN_FILEMUSTEXIST       = $00001000;
    OFN_EXPLORER            = $00080000;
    OFN_ENABLESIZING        = $00800000;
    OFN_FORCESHOWHIDDEN     = $10000000;    // Show All files including System and hidden files

//* Filesystem Dialogs messages
const

    FDM_NAME_SETTINGS_CHANGED = 'Filesystem Dialogs Settings Changed';

    //* Pointer to TOpenDialogResults is in wParam
    //* Pointer to TOpenDialogParams is in lParam
    //* You get the messages in the hookproc and to the specified Parent window handle
    FDM_BASE = 2 * WM_USER + 1;

    //* Filesystem Dialogs results message
    FDM_RESULTS = FDM_BASE + 1;

    //* Load settings, WPARAM = 0, LPARAM = 0
    FDM_LOADSETTINGS = FDM_BASE + 2;

    //* Free memory that you have allocated for the dialog('s params, results)
    FDM_FREEMEM = FDM_BASE + 3;

    //* The dialog form is closing
    FDM_CLOSING = FDM_BASE + 4;

    //* The dialog form's window handle in wParam, pointer to TOpenDialogParams is in lParam
    FDM_WINDOW_HANDLE = FDM_BASE + 5;

    //* Realing the integrated form to the parent
    FDM_REALIGN_INTEGRATED_WINDOW = FDM_BASE + 6;

    //* Sent on file name selection, file name is in wParam as PChar (unicode), item's PIDL in lParam
    FDM_ON_SELECT_FILE_NAME = FDM_BASE + 7;

    //* Refresh folder
    FDM_REFRESH_FOLDER = FDM_BASE + 8;

    //* Used internally
    FDM_THUMBNAIL_READY = FDM_BASE + 9;

    //* Used internally
    FDM_ADD_THUMBNAIL_FILENAME = FDM_BASE + 10;

    //* Used internally
    FDM_THUMBNAILER_THREAD_FINISHED = FDM_BASE + 11;

    //* Used internally
    FDM_THUMBNAILER_CONTROL_THREAD_TERMINATE = FDM_BASE + 12;

    //* Used internally
    FDM_SECONDARYVIEWER_PICTURE_READY = FDM_BASE + 13;

    //* Used internally
    FDM_SECONDARYVIEWER_AUDIOPICTURE_READY = FDM_BASE + 14;

    //* Used internally
    FDM_VIEWER_AUDIOPICTURE_READY = FDM_BASE + 15;

    //* Custom previewer query file, wParam = PChar file name, lParam = PIDL
    FDM_PREVIEWER_CUSTOM_QUERY = FDM_BASE + 16;
    
    //* Custom previewer open previewer, Wnd = handle for the panel, wParam = PChar file name, lParam = PIDL
    FDM_PREVIEWER_CUSTOM_DISPLAY = FDM_BASE + 17;
    
    //* Custom previewer close
    FDM_PREVIEWER_CUSTOM_CLOSE = FDM_BASE + 18;
    
    //* Custom previewer resized
    FDM_PREVIEWER_CUSTOM_RESIZED = FDM_BASE + 19;

    //* Used internally
    FDM_VIEWER_AUDIOPICTURE_PROGRESS = FDM_BASE + 20;

    //* Used internally
    FDM_SECONDARYVIEWER_VIEWER_AUDIOPICTURE_PROGRESS = FDM_BASE + 21;
    
    //* Navigate to folder specified as PIDL in WPARAM, LPARAM = 0
    FDM_NAVIGATE_TO_PIDL = FDM_BASE + 22;

    //* Trigger sending of selected files and folders for hookproc, WPARAM = 0, LPARAM = 0
    FDM_QUERY_SELECTED_ITEMS = FDM_BASE + 23;

    //* Abort (stop) listing a folder, WPARAM = 0, LPARAM = 0
    FDM_CANCEL_LISTING_ITEMS = FDM_BASE + 24;
    
    //* Used internally
    FDM_VIEWER_GIFANIM_READY = FDM_BASE + 25;

    //* Used internally
    FDM_SECONDARYVIEWER_GIFANIM_READY = FDM_BASE + 26;    

    //* Request the current folder, returns POpenDialogOnCurrentFolder in WPARAM, POpenDialogParams in LPARAM
    FDM_GET_CURRENT_FOLDER = FDM_BASE + 27;

    //* A dialog window is activated, returns 0 in WPARAM, POpenDialogParams in LPARAM
    FDM_WINDOW_ACTIVATED = FDM_BASE + 28;

    //* Activate a dialog window, WPARAM = 0, LPARAM = 0
    FDM_WINDOW_ACTIVATE = FDM_BASE + 29;

    //* Select all files and folders, WPARAM = 0, LPARAM = 0
    FDM_SELECT_ALL = FDM_BASE + 30;

    //* Invert selected and unselected files and folders, WPARAM = 0, LPARAM = 0
    FDM_INVERT_SELECTION = FDM_BASE + 31;

    //* Invert selected and unselected files and folders, WPARAM = 0, LPARAM = POpenDialogSelectPattern
    FDM_SELECT_PATTERN = FDM_BASE + 32;

    //* Current folder changed, WPARAM = PIDL, LPARAM = POpenDialogParams
    FDM_CURRENT_FOLDER_CHANGED = FDM_BASE + 33;

    //* Editing text, WPARAM = 1 when started, 0 when exited, LPARAM = POpenDialogSelectPattern
    FDM_IN_EDIT = FDM_BASE + 34;

    //* Deselect all files and folders, WPARAM = 0, LPARAM = 0
    FDM_SELECT_NONE = FDM_BASE + 35;

    //* Used internally
    FDM_FOLDER_SIZE_RESULT = FDM_BASE + 36;

    //* Get folder sizes, WPARAM = 0, LPARAM = 0
    FDM_COUNT_FOLDER_SIZES = FDM_BASE + 37;

    //* Display the search panel, WPARAM = 0, LPARAM = 0
    FDM_SHOW_SEARCH_PANEL = FDM_BASE + 38;

    //* Add a PIDL, WPARAM = PIDL, LPARAM = 0
    FDM_ADD_PIDL_TO_LIST = FDM_BASE + 39;

    //* Search completed, WPARAM = 0, LPARAM = 0
    FDM_SEARCH_COMPLETED = FDM_BASE + 40;

    //* Search completed, WPARAM = TFDSearchProgress, LPARAM = 0
    FDM_SEARCH_PROGRESS = FDM_BASE + 41;

    //* Style changed, WPARAM = TFDStyleChanged, LPARAM = 0
    FDM_STYLE_CHANGED = FDM_BASE + 42;

    //* Host, parent window, handle changed, WPARAM = NewHandle, LPARAM = 0
    FDM_HOST_HANDLE_CHANGED = FDM_BASE + 43;

    //* Reference to dialog class instance, WPARAM = TFormFileDialog, LPARAM = POpenDialogParams
    FDM_DIALOG_INSTANCE = FDM_BASE + 44;

    //* Pop-up settings menu, WPARAM = X, LPARAM = Y
    FDM_DIALOG_POPUP_SETTINGS_MENU = FDM_BASE + 45;

Const
    FD_DW_DEFAULT = $00000000;

type
    TOpenDialogAppearance = record
        UIColor: Cardinal;                              //* ARGB (use FD_DW_DEFAULT = no change)
        UIFontColor: Cardinal;                          //* ARGB
        ListerBckgColor: Cardinal;                      //* ARGB
        ListerFontColor: Cardinal;                      //* ARGB
        ListerGridLines: LongBool;
        ListerHeaderGradientTopColor: Cardinal;         //* ARGB
        ListerHeaderGradientBottomColor: Cardinal;      //* ARGB
        ListerHeaderBrightColor: Cardinal;              //* ARGB
        ListerHeaderDarkColor: Cardinal;                //* ARGB
        ListerHeaderFontColor: Cardinal;                //* ARGB
        ListerSelectionAlpha: Byte;                     //* A
        ListerSelectionGradientTopColor: Cardinal;      //* ARGB
        ListerSelectionGradientBottomColor: Cardinal;   //* ARGB
        ListerSelectionFrameColor: Cardinal;            //* ARGB
        ListerHotTrackAlpha: Byte;                      //* A
        ListerHotTrackGradientTopColor: Cardinal;       //* ARGB
        ListerHotTrackGradientBottomColor: Cardinal;    //* ARGB
        ListerHotTrackFrameColor: Cardinal;             //* ARGB
        VCLStyleTheme: PChar;                           //* For example 'AQUA GRAPHITE' or a file name of a Delphi style file 'C:\Carbon.vsf'
    end;
    POpenDialogAppearance = ^TOpenDialogAppearance;

//* Filesystem dialogs types
type
    TFSDialogType = (fsdtOpenDialog, fsdtSaveDialog, fsdtFolderDialog);

//* Filesystem dialog Open file/Save file dialog Params
type
    TOpenDialogParams = record
        DefaultExt: PWideChar;                          //* Default extension
        FileName: PWideChar;                            //* Default file name for Open/Save and Initial dir
        Filters: PWideChar;                             //* Filters eg.: 'Image files (*.bmp,*.jpg)|*.bmp;*.jpg|All files|*.*'
        FilterIndex: Integer;                           //* Filter to use default, 1-first, 2-second...
        HelpContext: Integer;                           //* Help context (not yet implemented)
        InitialDir: PWideChar;                          //* Initial folder, this is extracted first from FileName, if no FileName specified settings are used to decide which folder to use
        Options: TOpenOptions;                          //* Dialog options (not all implemented so far), OR TOpenOption flags together
        OptionsEx: TOpenOptionsEx;                      //* Extra Dialog options, specify panels to hide (not all implemented), OR TOpenOptionEx flags together
        Title: PWideChar;                               //* Title to display as window's caption
        ParentWindow: HWND;                             //* Parent window handle (needed for HookProc)
        DialogGUID: TGUID;                              //* Specifically identificate the dialog (separate dialogs inside an application - you can get the same dialog always with this flag)
        HookProc: OFNHookProc;                          //* Hookproc (There are two: 1. for not modal mode result, and 2. for filter change event (avoid using it) - ParentWindow handle is also needed)
        DialogType: TFSDialogType;                      //* Type of the dialog: Open/Save/Folder
        User: Pointer;                                  //* User variable
        FileInfoHookProc: FDFileInfoHookProc;           //* Callback that gives back custom file info
        FileInfoColumnName: PWideChar;                  //* Column name used for custom file info
        FileThumbnailHookProc: FDFileThumbnailHookProc; //* Callback that gives back thumbnail
        Appearance: POpenDialogAppearance;              //* Set to nil to use default settings
        AdditionalParameters: PChar;                    //* String list of additional parameters
    end;
    POpenDialogParams = ^TOpenDialogParams;

//* Filesystem dialog result
type
    TOpenDialogResults = record
        Successfull: BOOL;          //* True: User pressed the Open/Save button or double-clicked on a file, False: User pressed Cancel or closed the window, or error (there is fallback to comdlg32.dll though)
        FileName: PChar;            //* The filename with full path the user selected (only one) ending with a double null: "C:\Dir\Data.txt" + #0#0
        FileNameSize: Int64;        //* FileName size in bytes (should be always at least MAX_PATH)
        FileCount: Cardinal;        //* Number of files, if user selected multiple files then FileCount > 1
        FileNames: PChar;           //* List of files in format "C:\Dir\Data.txt" + #0 + "D:\Folder\DataX.txt" separated by null characters ending with a double null
        FileNamesSize: Int64;       //* FileNames size in bytes (it's up to you to validate last entry if all selected filenames don't fit into the FileNames - but always check with FileExists())
        FilterIndex: Integer;       //* Filter's index the user selected 1-first, 2-second, etc...
    end;
    POpenDialogResults = ^TOpenDialogResults;

//* FDM_ON_SELECT_FILE_NAME in WPARAM
type
    TOpenDialogOnSelectFileName = record
        FileName: PChar;            //* The filename with full path the user selected (only one)
        Item: PItemIDList;          //* File' PIDL
    end;
    POpenDialogOnSelectFileName = ^TOpenDialogOnSelectFileName;

//* FDM_GET_CURRENT_FOLDER in WPARAM
type
    TOpenDialogOnCurrentFolder = record
        Folder: PChar;              //* The full path the user selected (only one)
        Item: PItemIDList;          //* Folder's PIDL
    end;
    POpenDialogOnCurrentFolder = ^TOpenDialogOnCurrentFolder;

//* FDM_SELECT_PATTERN in LPARAM
type
    TOpenDialogSelectPattern = record
        Pattern: PChar;             //* Pattern string
        CaseSensitive: LongBool;    //*
    end;
    POpenDialogSelectPattern = ^TOpenDialogSelectPattern;

type
    TFDSearchProgress = record
        FileName: String;
    end;
    PFDSearchProgress = ^TFDSearchProgress;

//* FDM_STYLE_CHANGED in WPARAM
type
    TFDStyleChanged = record
        StyleName: String;
    end;
    PFDStyleChanged = ^TFDStyleChanged;

type
    TFDStyle = (
        fdsWindows,
        fdsAmakrits,
        fdsAmethystKamri,
        fdsAquaGraphite,
        fdsAquaLightSlate,
        fdsAuric,
        fdsCarbon,
        fdsCharcoalDarkSlate,
        fdsCobaltXEMedia,
        fdsCyanDusk,
        fdsCyanNight,
        fdsEmeraldLightSlate,
        fdsGlossy,
        fdsGlow,
        fdsGoldenGraphite,
        fdsIcebergClassico,
        fdsLavenderClassico,
        fdsLight,
        fdsLuna,
        fdsObsidian,
        fdsOnyxBlue,
        fdsRubyGraphite,
        fdsSapphireKamri,
        fdsSilver,
        fdsSky,
        fdsSlateClassico,
        fdsSmokeyQuartzKamri,
        fdsTabletDark,
        fdsTabletLight,
        fdsTurquoiseGray,
        fdsWindows10,
        fdsWindows10Blue,
        fdsWindows10Dark,
        fdsWindows10Green,
        fdsWindows10Purple,
        fdsWindows10SlateGray
    );
    TFDStyles = set of TFDStyle;

//* Filesystem Dialogs settings class
type
    TFilesystemDialogsSettings = class(TComponent)
    private
    public
        UseLastFolder: Boolean;
        SingleSetup: Boolean;
        FileDateTimeMSPrecision: Boolean;
        Daynames: Boolean;
        AutoGoIntoCreatedFolder: Boolean;
        SaveSettingsOnCancel: Boolean;
        ThumbnailsForDirs: Boolean;
        ThumbnailsForDirsAudio: Boolean;
        ThumbnailsForAudioFiles: Boolean;
        ThumbnailsForAudioFilesShadow: Boolean;
        ThumbnailsForTextFiles: Boolean;
        ThumbnailsGIFAnim: Boolean;
        ThumbnailsSmallZoomx2: Boolean;
        ThumbnailsWidth: Integer;
        ThumbnailsHeight: Integer;
        ThumbnailsBalloonWidth: Integer;
        ThumbnailsBalloonHeight: Integer;
        ThumbnailsSkeezWidth: Integer;
        ThumbnailsSkeezHeight: Integer;
        ThumbnailsSizePreset: Integer;
        ThumbnailsSampleColorL: Cardinal;
        ThumbnailsSampleColorR: Cardinal;
        ThumbnailsSampleColorB: Cardinal;
        ThumbnailsSampleColorLViewer: Cardinal;
        ThumbnailsSampleColorRViewer: Cardinal;
        ThumbnailsSampleColorBViewer: Cardinal;
        ThumbnailsSampleAALEvel: Integer;
        ThumbnailsFolderIconColor: Integer;
        ThumbnailsCachePicture: Boolean;
        ThumbnailsCacheAudio: Boolean;
        ThumbnailsThreadsAll: Boolean;
        SaveSESC: Boolean;
        ForceExtCase: Boolean;
        ForceExtCaseUpper: Boolean;
        ForceExtCaseLower: Boolean;
        Font: TFont;
        InactiveTreeViewUntilFirstTouch: Boolean;
        OmitFileExt: Boolean;
        DisplayDetails: Boolean;
        SettingsWindowPosition: TPoint;
        SettingsPageIndex: Integer;
        SettingsPageThumbsIndex: Integer;
        SettingsPagePrefsIndex: Integer;
        RecentFolderCount: Int64;
        RecentFileCount: Int64;
        ShowScrollbarsRecent: Boolean;
        ShowScrollbarsTreeView: Boolean;
        ConvertDisplayExtensions: Boolean;
        UseSecondaryViewer: Boolean;
        SecondaryViewerSpecified: Boolean;
        SecondaryViewerMonitorNo: Integer;
        SecondaryViewerDisplayOnHover: Boolean;
        SecondaryViewerAudio: Boolean;
        SecondaryViewerAudioNoProgress: Boolean;
        SecondaryViewerAudioNoInfo: Boolean;
        SecondaryViewerAudioAA: Integer;
        SecondaryViewerPicture: Boolean;
        SecondaryViewerPictureResampler: Integer;
        SecondaryViewerHTML: Boolean;
        SecondaryViewerHTMLAllFiles: Boolean;
        SecondaryViewerTEXTRTF: Boolean;
        BalloonInLister: Boolean;
        BalloonInThumbnailMode: Boolean;
        BalloonHintTimeOut: Integer;
        BalloonHasThumbs: Boolean;
        BASSDontInit: Boolean;
        TypesBrowser: String;
        TypesAudio: String;
        TypesText: String;
        TypesVideo: String;
        BalloonInLocations: Boolean;
        AutoConvertFirstLetterInFileNames: Boolean;
        AutoConvertUnderscoresInFileNames: Boolean;
        AutoURLDecodeFileNames: Boolean;
        DontDisplayTFDDB: Boolean;
        ShowHiddenSystem: Boolean;
        FilenameTemplate: String;
        AutoDelInvalidRecentLinks: Boolean;
        ThumbnailsForAudioFilesAsAPIC: Boolean;
        ThumbnailsForAudioFilesPreferCoverArts: Boolean;
        ThumbnailsForAudioFilesPreferSamples: Boolean;
        ThumbnailerTextZoom: Cardinal;
        NoCDN_TYPECHANGE: Boolean;
        TryWebBrowserForUnknownFileTypes: Boolean;
        ListerHasTheFocus: Boolean;
        NoBalloonWhenFileNameHasFocus: Boolean;
        UseCustomFontColor: Boolean;
        CustomFontColor: Cardinal;
        UseCustomBackgroundColor: Boolean;
        CustomBackgroundColor: Cardinal;
        DontAskIfNoFileTypeSpecified: Boolean;
        ThumbnailsUseWindowsFolderIcon: Boolean;
        UIFontSize: Integer;
        UIStyle: String;
        UIStyleWindowOverride: Boolean;
        UIStyleIntegratedOverride: Boolean;

        //* Skeez animation options
        SkeezAnimationIcon: Integer;
        SkeezAnimationIconX: Integer;
        SkeezAnimationIconY: Integer;
        SkeezAnimationIconDuration: Integer;

        SkeezAnimationFileName: Integer;
        SkeezAnimationFileNameX: Integer;
        SkeezAnimationFileNameY: Integer;
        SkeezAnimationFileNameDuration: Integer;

        SkeezAnimationFileSize: Integer;
        SkeezAnimationFileSizeX: Integer;
        SkeezAnimationFileSizeY: Integer;
        SkeezAnimationFileSizeDuration: Integer;

        SkeezAnimationDateTime: Integer;
        SkeezAnimationDateTimeX: Integer;
        SkeezAnimationDateTimeY: Integer;
        SkeezAnimationDateTimeDuration: Integer;

        SkeezAnimationDescription: Integer;
        SkeezAnimationDescriptionX: Integer;
        SkeezAnimationDescriptionY: Integer;
        SkeezAnimationDescriptionDuration: Integer;

        SkeezAnimationDetails: Integer;
        SkeezAnimationDetailsX: Integer;
        SkeezAnimationDetailsY: Integer;
        SkeezAnimationDetailsDuration: Integer;

        Constructor Create(AOwner: TComponent); override;
        Destructor Destroy; override;
        procedure LoadSettings;
        procedure SaveSettings(AnimationOptionsToo: Boolean);
    end;

//* Exported functions
type
    t_OpenDialog            = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
    t_SaveDialog            = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
    t_FolderDialog          = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;

    t_FilesystemDialogsSettings = function(Params: PChar): BOOL; stdcall;

    t_FreeDialogResults     = function(var Struct: TOpenDialogResults): Boolean; stdcall;

    t_GetFileName           = function(var OpenFileParams: tagOFN): BOOL; stdcall;

    t_GetFolderA            = function(var lpbi: TSHBrowseinfoA): PItemIDList; stdcall;
    t_GetFolderW            = function(var lpbi: TSHBrowseinfoW): PItemIDList; stdcall;

    t_GetPrevProcs          = function(var pGetOpenFileNameA, pGetOpenFileNameW, pGetSaveFileNameA, pGetSaveFileNameW: t_GetFileName; var pSHBrowseForFolderA: t_GetFolderA; var pSHBrowseForFolderW: t_GetFolderW): Boolean; register;

type
    TFilesystemDialogsLastSettings = record
        Folder: String;
        FilterIndex: Integer;
    end;

type
    TFilesystemDialog = class;

    TResultsEvent = procedure(Sender: TObject; var FileName: String; var Files: TStrings) of object;
    TFilesystemDialogOnSelectFileNameEvent = procedure (Sender: TFilesystemDialog; FileName: String; PIDL: PItemIDList) of object;
    TFilesystemDialogOnSelectFileNamesEvent = procedure (Sender: TFilesystemDialog; FileNames: TStringList; PIDL: PItemIDList) of object;

    //* Delphi component layer (compatible with TOpenDialog and TSaveDialog, that means you can change the type name from
    //* TOpen/SaveDialog to TFilesystemDialog and it should work. To access the full functionality use the dynamic loading with direct calls 'FilesystemDialogsDynamic.pas'.)

    {$IF CompilerVersion >= 23}
    [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
    {$IFEND}
    TFilesystemDialog = class(TComponent)
    private
        fDefaultExt: String;
        fFileName: String;
        fFilter: String;
        fInitialDir: String;
        fPath: String;
        fTitle: String;
        fOptions: TOpenOptions;
        fOptionsEx: TOpenOptionsEx;
        fOnSelectionChange: TNotifyEvent;
        fOnFolderChange: TNotifyEvent;
        fOnTypeChange: TNotifyEvent;
        fOnCanClose: TCloseQueryEvent;
        fOnIncludeItem: TIncludeItemEvent;
        fDialogType: TFSDialogType;
        fFiles: TStrings;
        fMaxFiles: Cardinal;
        fGUID: TGUID;
        fResultsEvent: TResultsEvent;
        FStyle: TFDStyle;
        FUsedStyleString: String;
        FOnSelectFileNameEvent: TFilesystemDialogOnSelectFileNameEvent;
        FOnSelectFileNamesEvent: TFilesystemDialogOnSelectFileNamesEvent;
        FUIDPIScaling: Cardinal;
        FUIDPIScalingString: String;
        function GetFileName: TFileName;
        procedure SetFileName(const FileName: TFileName);
        function GetFilterIndex: Integer;
        procedure SetFilterIndex(Index: Integer);
        function GetOptions: TOpenOptions;
        procedure SetOptions(OpenOptions: TOpenOptions);
        function GetOptionsEx: TOpenOptionsEx;
        procedure SetOptionsEx(OpenOptionsEx: TOpenOptionsEx);
        function GetGuid: String;
        procedure SetGuid(const Value: String);
        function GetLastUsedSettings: TFilesystemDialogsLastSettings;
        procedure SetLastUsedSettings(LastSettings: TFilesystemDialogsLastSettings);
    protected
        //*
    public
        Params: TOpenDialogParams;
        Results: TOpenDialogResults;
        DefWinOpenDialog: TOpenDialog;
        DefWinSaveDialog: TSaveDialog;
        FilesystemDialogHandle: THandle;
        function LoadFilesystemDialogsLibrary: Boolean;
        property Files: TStrings read fFiles;
        Constructor Create(AOwner: TComponent); override;
        Destructor Destroy; override;
        function Execute: Boolean;
        function FreeTOpenDialogResults(var Struct: TOpenDialogResults): Boolean;
        function Settings: Boolean;
        function CallDefWinDialog: Boolean;
        function GetUseStyle: PChar;
    published
        property DialogType: TFSDialogType read fDialogType write fDialogType default fsdtOpenDialog;
        property DefaultExt: String read fDefaultExt write fDefaultExt;
        property FileName: TFileName read GetFileName write SetFileName;
        property Filter: String read fFilter write fFilter;
        property FilterIndex: Integer read GetFilterIndex write SetFilterIndex default 1;
        property InitialDir: String read fInitialDir write fInitialDir;
        property Path: String read fPath;
        property Options: TOpenOptions read GetOptions write SetOptions;
        property OptionsEx: TOpenOptionsEx read GetOptionsEx write SetOptionsEx;
        property DialogTitle: String read fTitle write fTitle;
        property OnCanClose: TCloseQueryEvent read fOnCanClose write fOnCanClose;
        property OnFolderChange: TNotifyEvent read fOnFolderChange write fOnFolderChange;
        property OnSelectionChange: TNotifyEvent read fOnSelectionChange write fOnSelectionChange;
        property OnTypeChange: TNotifyEvent read fOnTypeChange write fOnTypeChange;
        property OnIncludeItem: TIncludeItemEvent read fOnIncludeItem write fOnIncludeItem;
        property OnResults: TResultsEvent read fResultsEvent write fResultsEvent;
        property MaxFiles: Cardinal read fMaxFiles write fMaxFiles;
        property DialogGUID: String read GetGUID write SetGUID;
        property Style: TFDStyle read FStyle write FStyle;
        property OnSelectFileName: TFilesystemDialogOnSelectFileNameEvent read FOnSelectFileNameEvent write FOnSelectFileNameEvent;
        property OnSelectFileNames: TFilesystemDialogOnSelectFileNamesEvent read FOnSelectFileNamesEvent write FOnSelectFileNamesEvent;
        property UIDPIScaling: Cardinal read FUIDPIScaling write FUIDPIScaling default 0;
    end;

    procedure Register;

    function TFDStyle2String(FDStyle: TFDStyle): String;

var
    FilesystemDialogsDLLHandle: THandle;
    FilesystemDialogsOpenDialog: t_OpenDialog;
    FilesystemDialogsSaveDialog: t_SaveDialog;
    FilesystemDialogsFolderDialog: t_FolderDialog;
    FilesystemDialogsFreeDialogResults: t_FreeDialogResults;
    FilesystemDialogsSettings: t_FilesystemDialogsSettings;
    FilesystemDialogsGlobalStyle: String;
    FilesystemDialogsGlobalUIDPI: Cardinal;
    FilesystemDialogsGlobalUseWindowsDialogs: Boolean = False;
    FilesystemDialogsDoNotLoadFilesystemDialogsDLL: Boolean;

implementation

Uses
    Registry,
    LogicalProcessorInformation;

procedure Register;
begin
    RegisterComponents('3delite', [TFilesystemDialog]);
end;

//* Taken from Graphics32
function Color32(WinColor: TColor): TColor32;
begin
    if WinColor < 0 then WinColor := GetSysColor(WinColor and $000000FF);
    Result := $FF shl 24 + (WinColor and $FF0000) shr 16 + (WinColor and $FF00) + (WinColor and $FF) shl 16;
end;

function Bytes2MB(Bytes: Int64): String;
begin
    if Bytes > 1048576 then begin
        if Bytes > 1073741824
            then Result := FloatToStrF((Bytes / 1073741824), ffFixed, 4, 2) + ' GB'
            else Result := FloatToStrF((Bytes / 1048576), ffFixed, 4, 2) + ' MB';
    end else Result := FloatToStrF((Bytes / 1024), ffFixed, 4, 2) + ' KB';
    if Bytes < 1024
 	    then Result := IntToStr(Bytes) + ' Bytes';
end;

function RuningOn64BitOS: Boolean;
type
    TIsWow64Process = function(Handle:THandle; var IsWow64 : BOOL) : BOOL; stdcall;
var
    hKernel32: HMODULE;
    IsWow64Process: TIsWow64Process;
    IsWow64: BOOL;
begin
    // we can check if the operating system is 64-bit by checking whether
    // we are running under Wow64 (we are 32-bit code). We must check if this
    // function is implemented before we call it, because some older versions
    // of kernel32.dll (eg. Windows 2000) don't know about it.
    // see http://msdn.microsoft.com/en-us/library/ms684139%28VS.85%29.aspx
    Result := False;
    hKernel32 := LoadLibrary('kernel32.dll');
    if hKernel32 <> 0 then begin
        @IsWow64Process := GetProcAddress(hkernel32, 'IsWow64Process');
        if Assigned(IsWow64Process) then begin
            IsWow64 := False;
        end;
        if (IsWow64Process(GetCurrentProcess, IsWow64)) then begin
            Result := IsWow64;
        end;
        FreeLibrary(hKernel32);
    end;
end;

function FDHookProc(Wnd: HWND; Msg: UINT; W_Param: WPARAM; L_Param: LPARAM): UINT stdcall;
var
    Results: POpenDialogResults;
    FileName: String;
    Struct: Pointer;
    PFileName: PChar;
    FilesystemDialogInstance: TFilesystemDialog;
    FileNames: TStringList;
begin
    //* Results arrived
    if Msg = FDM_RESULTS then begin
        Results := POpenDialogResults(W_Param);
        FilesystemDialogInstance := TFilesystemDialog(POpenDialogParams(Pointer(L_Param)).User);
        //* Extract files (multiple) ending with a double null
        FileNames := TStringList.Create;
        try
            if Results.FileCount = 1 then begin
                FileName := Results.FileName;
                FileNames.Append(FileName);
            end else if Results.FileCount > 1 then begin
                PFileName := Results.FileNames;
                while PFileName^ <> #0 do begin
                    FileName := PFileName;
                    FileNames.Append(FileName);
                    Inc(PFileName, Length(PFileName) + 1);
                end;
            end;
            FilesystemDialogInstance.Files.Text := FileNames.Text;
            if FileNames.Count > 0 then begin
                FilesystemDialogInstance.FileName := FileNames[0];
                if Assigned(FilesystemDialogInstance.OnSelectFileNames) then begin
                    FilesystemDialogInstance.OnSelectFileNames(FilesystemDialogInstance, FileNames, nil);
                end;
            end;
        finally
            FreeAndNil(FileNames);
        end;
    end;
    //* User selected a file
    if Msg = FDM_ON_SELECT_FILE_NAME then begin
        FilesystemDialogInstance := TFilesystemDialog(POpenDialogParams(Pointer(L_Param)).User);
        if Assigned(FilesystemDialogInstance.OnSelectFileName) then begin
            FilesystemDialogInstance.OnSelectFileName(FilesystemDialogInstance, POpenDialogOnSelectFileName(W_Param).FileName, POpenDialogOnSelectFileName(W_Param).Item);
        end;
    end;
    //* Free the memory as it is not needed anymore
    if Msg = FDM_FREEMEM then begin
        Struct := Pointer(W_Param);
        FilesystemDialogInstance := TFilesystemDialog(POpenDialogParams(Pointer(L_Param)).User);
        //* Free our appearance record if we have one
        if Assigned(POpenDialogParams(L_PARAM).Appearance) then begin
            if Assigned(POpenDialogParams(L_PARAM).Appearance.VCLStyleTheme) then begin
                FreeMem(POpenDialogParams(L_PARAM).Appearance.VCLStyleTheme);
            end;
            Dispose(POpenDialogParams(L_PARAM).Appearance);
        end;
        //* Free memory
        FilesystemDialogInstance.FreeTOpenDialogResults(TOpenDialogResults(Struct^));
    end;
    //* The window handle arrived
    if Msg = FDM_WINDOW_HANDLE then begin
        FilesystemDialogInstance := TFilesystemDialog(POpenDialogParams(Pointer(L_Param)).User);
        FilesystemDialogInstance.FilesystemDialogHandle := W_Param;
    end;
    Result := 1;
end;

constructor TFilesystemDialog.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    DefWinOpenDialog := nil;
    DefWinSaveDialog := nil;
    if ComponentState = [csDesigning, csFreeNotification]
        then CreateGUID(fGUID);
    fMaxFiles := 100;
    fFiles := TStringList.Create;
    LoadFilesystemDialogsLibrary;
end;

destructor TFilesystemDialog.Destroy;
begin
    try
        FreeAndNil(fFiles);
        FreeAndNil(DefWinOpenDialog);
        FreeAndNil(DefWinSaveDialog);
    except
        //*
    end;
    try
        inherited Destroy;
    except
        //*
    end;
end;

function TFilesystemDialog.LoadFilesystemDialogsLibrary: Boolean;
begin
    Result := True;
    if FilesystemDialogsDLLHandle = 0 then begin
        if DefWinOpenDialog = nil
            then DefWinOpenDialog := TOpenDialog.Create(Self);
        if DefWinSaveDialog = nil
            then DefWinSaveDialog := TSaveDialog.Create(Self);
        Result := False;
    end;
end;

function TFilesystemDialog.Execute: Boolean;
begin

    Result := False;

    ZeroMemory(@Params, SizeOf(Params));
    ZeroMemory(@Results, SizeOf(Results));

    fFiles.Text := '';
    Results.FilterIndex := -1;

    if FilesystemDialogsGlobalUseWindowsDialogs then begin
        Result := CallDefWinDialog;
        Exit;
    end;

    //* Acquire library
    if FilesystemDialogsDLLHandle = 0 then begin
        if NOT LoadFilesystemDialogsLibrary then begin
            Result := CallDefWinDialog;
            Exit;
        end;
    end;

    //* Fill here
    Params.Options := fOptions;
    Params.OptionsEx := fOptionsEx;
    Params.DefaultExt := Pchar(fDefaultExt);
    Params.FileName := Pchar(fFileName);
    Params.Filters := Pchar(fFilter);
    Params.InitialDir := Pchar(fInitialDir);
    Params.Title := Pchar(fTitle);
    Params.DialogType := fDialogType;
    Params.DialogGUID := fGUID;

    Params.HookProc := FDHookProc;

    Params.User := Pointer(Self);

    if FilesystemDialogsGlobalStyle <> '' then begin
        New(Params.Appearance);
        Params.Appearance.VCLStyleTheme := AllocMem((Length(FilesystemDialogsGlobalStyle) + 1) * 2);
        StrLCopy(Params.Appearance.VCLStyleTheme, PWideChar(FilesystemDialogsGlobalStyle), Length(FilesystemDialogsGlobalStyle));
    end else begin
        if FStyle <> fdsWindows then begin
            New(Params.Appearance);
            Params.Appearance.VCLStyleTheme := AllocMem((Length(TFDStyle2String(FStyle)) + 1) * 2);
            StrLCopy(Params.Appearance.VCLStyleTheme, PWideChar(TFDStyle2String(FStyle)), Length(TFDStyle2String(FStyle)));
        end;
    end;

    if FUIDPIScaling <> 0 then begin
        FUIDPIScalingString := 'UIDPIScaling=' + IntToStr(FUIDPIScaling);
        Params.AdditionalParameters := PChar(FUIDPIScalingString);
    end else if FilesystemDialogsGlobalUIDPI <> 0 then begin
        FUIDPIScalingString := 'UIDPIScaling=' + IntToStr(FilesystemDialogsGlobalUIDPI);
        Params.AdditionalParameters := PChar(FUIDPIScalingString);
    end;

    //* Execute
    case Params.DialogType of
        fsdtOpenDialog: Result := FilesystemDialogsOpenDialog(Params, Results);
        fsdtSaveDialog: Result := FilesystemDialogsSaveDialog(Params, Results);
        fsdtFolderDialog: Result := FilesystemDialogsFolderDialog(Params, Results);
    end;

end;

function GetFileNameFromPIDL(Item: PItemIDList): String;
var
    FBuf: array[0..MAX_PATH] of Char;
begin
    SHGetPathFromIDList(Item, @FBuf[0]);
    Result := FBuf;
end;

function GetPIDLFromFileName(FileName: String): PItemIDList;
var
    Desktop: IShellFolder;
    OleStr: array[0..MAX_PATH] of WideChar;
    CardOut: Cardinal;
begin
     Result := nil;
     if Failed(SHGetDesktopFolder(Desktop)) then begin
        Exit;
     end;
     //piDesktop._AddRef;
     CopyMemory(@OleStr[0], @FileName[1], (Length(FileName) + 1) * 2);
     Desktop.ParseDisplayName(0, nil, @OleStr, CardOut, Result, CardOut);
end;

function BrowseForFolder(Handle: THandle; Title: String; Folder: String): String;
var
    WindowList: Pointer;
    bi: TBrowseInfo;
    pidl: PItemIDList;
    strPath: PChar;
begin
    FillChar(bi, SizeOf(bi), 0);
    bi.hwndOwner := Handle;
    //if DirectoryExists(Folder) then begin
    //    bi.pidlRoot := GetPIDLFromFileName(Folder);
    //end else begin
        bi.pidlRoot := nil;
    //end;
    bi.pszDisplayName := nil;
    bi.lpszTitle := PChar(Title);
    bi.ulFlags := BIF_RETURNONLYFSDIRS OR BIF_NEWDIALOGSTYLE;
    bi.lpfn := nil;
    bi.lParam := 0;
    WindowList := DisableTaskWindows(0);
    Try
        pidl := ShBrowseForFolder(bi);
    finally
        EnableTaskWindows(WindowList);
        if Assigned(bi.pidlRoot) then begin
            ILFree(bi.pidlRoot);
        end;
    end;
    if Assigned(pidl) then begin
        strPath := AllocMem(MAX_PATH * 2);
        ShGetPathFromIdList(pidl, strPath);
        Result := strPath;
        Result := IncludeTrailingPathDelimiter(Result);
        ILFree(pidl);
        FreeMem(strPath);
    end else begin
        Result := '';
    end;
end;

function TFilesystemDialog.CallDefWinDialog: Boolean;
var
    LastUsedSettings: TFilesystemDialogsLastSettings;
begin
    Result := False;
    try
        if DefWinOpenDialog = nil
            then DefWinOpenDialog := TOpenDialog.Create(Self);
        if DefWinSaveDialog = nil
            then DefWinSaveDialog := TSaveDialog.Create(Self);

        LastUsedSettings := GetLastUsedSettings;

        //* Execute
        case fDialogType of
            fsdtOpenDialog: begin
                if DefWinOpenDialog <> nil then begin
                    DefWinOpenDialog.Options := [];
                    if ofReadOnly in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofReadOnly];
                    end;
                    if ofOverwritePrompt in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofOverwritePrompt];
                    end;
                    if ofHideReadOnly in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofHideReadOnly];
                    end;
                    if ofNoChangeDir in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofNoChangeDir];
                    end;
                    if ofShowHelp in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofShowHelp];
                    end;
                    if ofNoValidate in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofNoValidate];
                    end;
                    if ofAllowMultiSelect in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofAllowMultiSelect];
                    end;
                    if ofExtensionDifferent in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofExtensionDifferent];
                    end;
                    if ofPathMustExist in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofPathMustExist];
                    end;
                    if ofFileMustExist in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofFileMustExist];
                    end;
                    if ofCreatePrompt in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofCreatePrompt];
                    end;
                    if ofShareAware in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofShareAware];
                    end;
                    if ofNoReadOnlyReturn in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofNoReadOnlyReturn];
                    end;
                    if ofNoTestFileCreate in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofNoTestFileCreate];
                    end;
                    if ofNoNetworkButton in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofNoNetworkButton];
                    end;
                    if ofNoDereferenceLinks in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofNoDereferenceLinks];
                    end;
                    if ofEnableIncludeNotify in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofEnableIncludeNotify];
                    end;
                    if ofDontAddToRecent in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofDontAddToRecent];
                    end;
                    if ofForceShowHidden in fOptions then begin
                        DefWinOpenDialog.Options := DefWinOpenDialog.Options + [Dialogs.TOpenOption.ofForceShowHidden];
                    end;
                    DefWinOpenDialog.DefaultExt := fDefaultExt;
                    DefWinOpenDialog.FileName := fFileName;
                    DefWinOpenDialog.Filter := fFilter;
                    if fInitialDir <> '' then begin
                        DefWinOpenDialog.InitialDir := fInitialDir;
                    end else begin
                        DefWinOpenDialog.InitialDir := LastUsedSettings.Folder;
                    end;
                    if Self.FilterIndex > - 1 then begin
                        DefWinOpenDialog.FilterIndex := Self.FilterIndex + 1;
                    end else begin
                        DefWinOpenDialog.FilterIndex := LastUsedSettings.FilterIndex;
                    end;
                    DefWinOpenDialog.Title := fTitle;
                    Result := DefWinOpenDialog.Execute;
                    if Result then begin
                        Self.FileName := DefWinOpenDialog.FileName;
                        Params.FileName := PChar(DefWinOpenDialog.FileName);
                        fPath := ExtractFilePath(DefWinOpenDialog.FileName);
                        fFiles.Text := DefWinOpenDialog.Files.Text;
                        Self.FilterIndex := DefWinOpenDialog.FilterIndex - 1;
                        Results.FileCount := DefWinOpenDialog.Files.Count;
                    end;
                end;
            end;
            fsdtSaveDialog: begin
                if DefWinSaveDialog <> nil then begin
                    DefWinSaveDialog.Options := [];
                    if ofReadOnly in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofReadOnly];
                    end;
                    if ofOverwritePrompt in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofOverwritePrompt];
                    end;
                    if ofHideReadOnly in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofHideReadOnly];
                    end;
                    if ofNoChangeDir in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofNoChangeDir];
                    end;
                    if ofShowHelp in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofShowHelp];
                    end;
                    if ofNoValidate in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofNoValidate];
                    end;
                    if ofAllowMultiSelect in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofAllowMultiSelect];
                    end;
                    if ofExtensionDifferent in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofExtensionDifferent];
                    end;
                    if ofPathMustExist in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofPathMustExist];
                    end;
                    if ofFileMustExist in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofFileMustExist];
                    end;
                    if ofCreatePrompt in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofCreatePrompt];
                    end;
                    if ofShareAware in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofShareAware];
                    end;
                    if ofNoReadOnlyReturn in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofNoReadOnlyReturn];
                    end;
                    if ofNoTestFileCreate in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofNoTestFileCreate];
                    end;
                    if ofNoNetworkButton in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofNoNetworkButton];
                    end;
                    if ofNoDereferenceLinks in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofNoDereferenceLinks];
                    end;
                    if ofEnableIncludeNotify in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofEnableIncludeNotify];
                    end;
                    if ofDontAddToRecent in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofDontAddToRecent];
                    end;
                    if ofForceShowHidden in fOptions then begin
                        DefWinSaveDialog.Options := DefWinSaveDialog.Options + [Dialogs.TOpenOption.ofForceShowHidden];
                    end;
                    DefWinSaveDialog.DefaultExt := fDefaultExt;
                    DefWinSaveDialog.FileName := fFileName;
                    DefWinSaveDialog.Filter := fFilter;
                    if fInitialDir <> '' then begin
                        DefWinSaveDialog.InitialDir := fInitialDir;
                    end else begin
                        DefWinSaveDialog.InitialDir := LastUsedSettings.Folder;
                    end;
                    if Self.FilterIndex > - 1 then begin
                        DefWinSaveDialog.FilterIndex := Self.FilterIndex + 1;
                    end else begin
                        DefWinSaveDialog.FilterIndex := LastUsedSettings.FilterIndex;
                    end;
                    DefWinSaveDialog.Title := fTitle;
                    Result := DefWinSaveDialog.Execute;
                    if Result then begin
                        Self.FileName := DefWinSaveDialog.FileName;
                        Params.FileName := PChar(DefWinSaveDialog.FileName);
                        fPath := ExtractFilePath(DefWinSaveDialog.FileName);
                        fFiles.Text := DefWinSaveDialog.Files.Text;
                        Self.FilterIndex := DefWinSaveDialog.FilterIndex - 1;
                        Results.FileCount := DefWinSaveDialog.Files.Count;
                    end;
                end;
            end;
            fsdtFolderDialog: begin
                if fInitialDir <> '' then begin
                    Self.FileName := BrowseForFolder(Application.Handle, Params.Title, '');
                end else begin
                    Self.FileName := BrowseForFolder(Application.Handle, Params.Title, LastUsedSettings.Folder);
                end;
                Params.FileName := PChar(Self.FileName);
                fPath := ExtractFilePath(Self.FileName);
                fFiles.Text := Self.FileName;
                Results.FileCount := 1;
                if Self.FileName <> '' then begin
                    Result := True;
                end else begin
                    Result := False;
                end;
            end;
        end;
        if Result then begin
            LastUsedSettings.Folder := fPath;
            LastUsedSettings.FilterIndex := FilterIndex;
            SetLastUsedSettings(LastUsedSettings);
        end;
    except
        //*
    end;
end;

function TFilesystemDialog.FreeTOpenDialogResults(var Struct: TOpenDialogResults): Boolean;
begin
    if NOT Assigned(FilesystemDialogsFreeDialogResults)
        then Result := False
        else Result := FilesystemDialogsFreeDialogResults(Struct);
end;

function TFilesystemDialog.Settings: Boolean;
begin
    if NOT Assigned(FilesystemDialogsSettings)
        then Result := False
        else Result := FilesystemDialogsSettings(nil);
end;

function TFilesystemDialog.GetFileName: TFileName;
begin
    Result := fFileName;
end;

procedure TFilesystemDialog.SetFileName(const FileName: TFileName);
begin
    fFileName := FileName;
end;

function TFilesystemDialog.GetFilterIndex: Integer;
begin
    Result := Results.FilterIndex;
end;

procedure TFilesystemDialog.SetFilterIndex(Index: Integer);
begin
    Results.FilterIndex := Index;
    Params.FilterIndex := Index;
end;

function TFilesystemDialog.GetOptions: TOpenOptions;
begin
    Result := fOptions;
end;

procedure TFilesystemDialog.SetOptions(OpenOptions: TOpenOptions);
begin
    fOptions := OpenOptions
end;

function TFilesystemDialog.GetOptionsEx: TOpenOptionsEx;
begin
    Result := fOptionsEx;
end;

procedure TFilesystemDialog.SetOptionsEx(OpenOptionsEx: TOpenOptionsEx);
begin
    fOptionsEx := OpenOptionsEx;
end;

function TFilesystemDialog.GetUseStyle: PChar;
begin
    FUsedStyleString := TFDStyle2String(FStyle);
    Result := PChar(FUsedStyleString);
end;

function TFilesystemDialog.GetGUID: String;
begin
    Result := GUIDToString(fGUID);
end;

function TFilesystemDialog.GetLastUsedSettings: TFilesystemDialogsLastSettings;
var
    Reg: TRegistry;
begin
    Result.Folder := '';
    Result.FilterIndex := 0;
    if RuningOn64BitOS then begin
        Reg := TRegistry.Create(KEY_WOW64_64KEY OR KEY_ALL_ACCESS);
    end else begin
        Reg := TRegistry.Create;
    end;
    try
        Reg.RootKey := HKEY_CURRENT_USER;
        if Reg.OpenKey(STR_REG_FD_SETTINGS + '\' + STR_REG_FD_DIALOG_SETTINGS + '\' + GUIDToString(fGUID), False) then begin
            if Reg.ValueExists('Folder') then begin
                Result.Folder := Reg.ReadString('Folder');
            end;
            if Reg.ValueExists('Filter index') then begin
                Result.FilterIndex := Reg.ReadInteger('Filter index');
            end;
        end;
    finally
        Reg.CloseKey;
        Reg.Free;
    end;
end;

procedure TFilesystemDialog.SetLastUsedSettings(LastSettings: TFilesystemDialogsLastSettings);
var
    Reg: TRegistry;
begin
    if RuningOn64BitOS then begin
        Reg := TRegistry.Create(KEY_WOW64_64KEY OR KEY_ALL_ACCESS);
    end else begin
        Reg := TRegistry.Create;
    end;
    try
        Reg.RootKey := HKEY_CURRENT_USER;
        if Reg.OpenKey(STR_REG_FD_SETTINGS + '\' + STR_REG_FD_DIALOG_SETTINGS + '\' + GUIDToString(fGUID), True) then begin
            if DirectoryExists(LastSettings.Folder) then begin
                Reg.WriteString('Folder', LastSettings.Folder);
            end;
            Reg.WriteInteger('Filter index', LastSettings.FilterIndex);
        end;
    finally
        Reg.CloseKey;
        Reg.Free;
    end;
end;

procedure TFilesystemDialog.SetGUID(const Value: String);
begin
    fGUID := StringToGUID(Value);
end;

//* TFileSystemDialogsSettings

constructor TFileSystemDialogsSettings.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    Font := TFont.Create;
    Daynames := True;
end;

destructor TFileSystemDialogsSettings.Destroy;
begin
    Font.Free;
    inherited Destroy;
end;

//* Load the global settings from the registry
//* This is the place also where the default settings are specified
procedure TFileSystemDialogsSettings.LoadSettings;
var
    Reg: TRegistry;
    LogicalProcessorInformation: TLogicalProcessorInformation;
begin
    if RuningOn64BitOS then begin
        Reg := TRegistry.Create(KEY_WOW64_64KEY OR KEY_ALL_ACCESS);
    end else begin
        Reg := TRegistry.Create;
    end;
    try
        Reg.RootKey := HKEY_CURRENT_USER;

        //* Global
        Reg.OpenKey(STR_REG_FD_SETTINGS + '\', False);

        if Reg.ValueExists(STR_REG_USE_LAST_FOLDER)
            then UseLastFolder := Reg.ReadBool(STR_REG_USE_LAST_FOLDER)
            else UseLastFolder := False;

        if Reg.ValueExists(STR_REG_SINGLE_SETUP)
            then SingleSetup := Reg.ReadBool(STR_REG_SINGLE_SETUP)
            else SingleSetup := False;

        if Reg.ValueExists(STR_REG_FILE_DATETIME_PRECISION)
            then FileDateTimeMSPrecision := Reg.ReadBool(STR_REG_FILE_DATETIME_PRECISION)
            else FileDateTimeMSPrecision := False;

        if Reg.ValueExists(STR_REG_READ_CREATED_FOLDER)
            then AutoGoIntoCreatedFolder := Reg.ReadBool(STR_REG_READ_CREATED_FOLDER)
            else AutoGoIntoCreatedFolder := True;

        if Reg.ValueExists(STR_REG_SAVE_SETTINGS_ON_CANCEL)
            then SaveSettingsOnCancel := Reg.ReadBool(STR_REG_SAVE_SETTINGS_ON_CANCEL)
            else SaveSettingsOnCancel := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_DIRS)
            then ThumbnailsForDirs := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_DIRS)
            else ThumbnailsForDirs := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_DIR_AUDIO)
            then ThumbnailsForDirsAudio := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_DIR_AUDIO)
            else ThumbnailsForDirsAudio := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_AUDIOFILES)
            then ThumbnailsForAudioFiles := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES)
            else ThumbnailsForAudioFiles := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_AUDIOFILES_SHADOW)
            then ThumbnailsForAudioFilesShadow := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_SHADOW)
            else ThumbnailsForAudioFilesShadow := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_TEXTFILES)
            then ThumbnailsForTextFiles := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_TEXTFILES)
            else ThumbnailsForTextFiles := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_GIF_ANIM)
            then ThumbnailsGIFAnim := Reg.ReadBool(STR_REG_THUMBNAILS_GIF_ANIM)
            else ThumbnailsGIFAnim := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_ZOOM_X2)
            then ThumbnailsSmallZoomx2 := Reg.ReadBool(STR_REG_THUMBNAILS_ZOOM_X2)
            else ThumbnailsSmallZoomx2 := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_WIDTH)
            then ThumbnailsWidth := Reg.ReadInteger(STR_REG_THUMBNAILS_WIDTH)
            else ThumbnailsWidth := 240;

        if Reg.ValueExists(STR_REG_THUMBNAILS_HEIGHT)
            then ThumbnailsHeight := Reg.ReadInteger(STR_REG_THUMBNAILS_HEIGHT)
            else ThumbnailsHeight := 150;

        if Reg.ValueExists(STR_REG_THUMBNAILS_BALLOON_WIDTH)
            then ThumbnailsBalloonWidth := Reg.ReadInteger(STR_REG_THUMBNAILS_BALLOON_WIDTH)
            else ThumbnailsBalloonWidth := 240;

        if Reg.ValueExists(STR_REG_THUMBNAILS_BALLOON_HEIGHT)
            then ThumbnailsBalloonHeight := Reg.ReadInteger(STR_REG_THUMBNAILS_BALLOON_HEIGHT)
            else ThumbnailsBalloonHeight := 240;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SKEEZ_WIDTH)
            then ThumbnailsSkeezWidth := Reg.ReadInteger(STR_REG_THUMBNAILS_SKEEZ_WIDTH)
            else ThumbnailsSkeezWidth := 140;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SKEEZ_HEIGHT)
            then ThumbnailsSkeezHeight := Reg.ReadInteger(STR_REG_THUMBNAILS_SKEEZ_HEIGHT)
            else ThumbnailsSkeezHeight := 120;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SIZE_PRESET)
            then ThumbnailsSizePreset := Reg.ReadInteger(STR_REG_THUMBNAILS_SIZE_PRESET)
            else ThumbnailsSizePreset := 2;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_COLOR_L)
            then ThumbnailsSampleColorL := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_L)
            else ThumbnailsSampleColorL := THUMBNAIL_COLOR_L;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_COLOR_R)
            then ThumbnailsSampleColorR := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_R)
            else ThumbnailsSampleColorR := THUMBNAIL_COLOR_R;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_COLOR_B)
            then ThumbnailsSampleColorB := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_B)
            else ThumbnailsSampleColorB := THUMBNAIL_COLOR_B;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_COLOR_L_VIEWER)
            then ThumbnailsSampleColorLViewer := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_L_VIEWER)
            else ThumbnailsSampleColorLViewer := THUMBNAIL_COLOR_1L;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_COLOR_R_VIEWER)
            then ThumbnailsSampleColorRViewer := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_R_VIEWER)
            else ThumbnailsSampleColorRViewer := THUMBNAIL_COLOR_1R;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_COLOR_B_VIEWER)
            then ThumbnailsSampleColorBViewer := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_B_VIEWER)
            else ThumbnailsSampleColorBViewer := THUMBNAIL_COLOR_1B;

        if Reg.ValueExists(STR_REG_THUMBNAILS_SAMPLE_AA)
            then ThumbnailsSampleAALEvel := Reg.ReadInteger(STR_REG_THUMBNAILS_SAMPLE_AA)
            else ThumbnailsSampleAALEvel := 4;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOLDER_COLOR)
            then ThumbnailsFolderIconColor := Reg.ReadInteger(STR_REG_THUMBNAILS_FOLDER_COLOR)
            else ThumbnailsFolderIconColor := 9;

        if Reg.ValueExists(STR_REG_THUMBNAILS_ALL_THREADS)
            then ThumbnailsThreadsAll := Reg.ReadBool(STR_REG_THUMBNAILS_ALL_THREADS)
            else begin
                LogicalProcessorInformation := GetLogicalProcessorInfo;
                if LogicalProcessorInformation.ProcessorCoreCount > 32 then begin
                    ThumbnailsThreadsAll := False;
                end else begin
                    ThumbnailsThreadsAll := True;
                end;
            end;

        if Reg.ValueExists(STR_REG_SAVE_SESC)
            then SaveSESC := Reg.ReadBool(STR_REG_SAVE_SESC)
            else SaveSESC := True;

        if Reg.ValueExists(STR_REG_FS_FORCE_EXTCASE)
            then ForceExtCase := Reg.ReadBool(STR_REG_FS_FORCE_EXTCASE)
            else ForceExtCase := False;

        if Reg.ValueExists(STR_REG_FS_FORCE_EXTCASEU)
            then ForceExtCaseUpper := Reg.ReadBool(STR_REG_FS_FORCE_EXTCASEU)
            else ForceExtCaseUpper := False;

        if Reg.ValueExists(STR_REG_FS_FORCE_EXTCASEL)
            then ForceExtCaseLower := Reg.ReadBool(STR_REG_FS_FORCE_EXTCASEL)
            else ForceExtCaseLower := False;

        if Reg.ValueExists(STR_REG_INACTIVE_TREEVIEW)
            then InactiveTreeViewUntilFirstTouch := Reg.ReadBool(STR_REG_INACTIVE_TREEVIEW)
            else InactiveTreeViewUntilFirstTouch := True;

        if Reg.ValueExists(STR_REG_OMIT_FILE_EXT)
            then OmitFileExt := Reg.ReadBool(STR_REG_OMIT_FILE_EXT)
            else OmitFileExt := False;

        if Reg.ValueExists(STR_REG_DETAILS)
            then DisplayDetails := Reg.ReadBool(STR_REG_DETAILS)
            else DisplayDetails := True;

        if Reg.ValueExists(STR_REG_SETTINGS_TOP)
            then SettingsWindowPosition.Y := Reg.ReadInteger(STR_REG_SETTINGS_TOP)
            else SettingsWindowPosition.Y := - 1;

        if Reg.ValueExists(STR_REG_SETTINGS_LEFT)
            then SettingsWindowPosition.X := Reg.ReadInteger(STR_REG_SETTINGS_LEFT)
            else SettingsWindowPosition.X := - 1;

        if Reg.ValueExists(STR_REG_SETTINGS_PAGE_INDEX)
            then SettingsPageIndex := Reg.ReadInteger(STR_REG_SETTINGS_PAGE_INDEX)
            else SettingsPageIndex := 0;

        if Reg.ValueExists(STR_REG_SETTINGS_PAGE_THUMBS_INDEX)
            then SettingsPageThumbsIndex := Reg.ReadInteger(STR_REG_SETTINGS_PAGE_THUMBS_INDEX)
            else SettingsPageThumbsIndex := 0;

        if Reg.ValueExists(STR_REG_SETTINGS_PAGE_PREFS_INDEX)
            then SettingsPagePrefsIndex := Reg.ReadInteger(STR_REG_SETTINGS_PAGE_PREFS_INDEX)
            else SettingsPagePrefsIndex := 0;

        if Reg.ValueExists(STR_REG_RECENT_FOLDER_COUNT)
            then RecentFolderCount := Reg.ReadInteger(STR_REG_RECENT_FOLDER_COUNT)
            else RecentFolderCount := 8;

        if Reg.ValueExists(STR_REG_SHOW_SCROLLBAR_RECENT)
            then ShowScrollbarsRecent := Reg.ReadBool(STR_REG_SHOW_SCROLLBAR_RECENT)
            else ShowScrollbarsRecent := True;

        if Reg.ValueExists(STR_REG_SHOW_SCROLLBAR_TREEVIEW)
            then ShowScrollbarsTreeView := Reg.ReadBool(STR_REG_SHOW_SCROLLBAR_TREEVIEW)
            else ShowScrollbarsTreeView := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_CACHED_GRAPHICAL)
            then ThumbnailsCachePicture := Reg.ReadBool(STR_REG_THUMBNAILS_CACHED_GRAPHICAL)
            else ThumbnailsCachePicture := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_CACHED_AUDIO)
            then ThumbnailsCacheAudio := Reg.ReadBool(STR_REG_THUMBNAILS_CACHED_AUDIO)
            else ThumbnailsCacheAudio := True;

        if Reg.ValueExists(STR_REG_SETTINGS_DISPLAY_EXTS_CONVERTED)
            then ConvertDisplayExtensions := Reg.ReadBool(STR_REG_SETTINGS_DISPLAY_EXTS_CONVERTED)
            else ConvertDisplayExtensions := False;

        if Reg.ValueExists(STR_REG_SEC_VIEWER)
            then UseSecondaryViewer := Reg.ReadBool(STR_REG_SEC_VIEWER)
            else UseSecondaryViewer := False;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_SPECIFY)
            then SecondaryViewerSpecified := Reg.ReadBool(STR_REG_SEC_VIEWER_SPECIFY)
            else SecondaryViewerSpecified := False;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_MONITOR)
            then SecondaryViewerMonitorNo := Reg.ReadInteger(STR_REG_SEC_VIEWER_MONITOR)
            else SecondaryViewerMonitorNo := 0;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_ONHOVER)
            then SecondaryViewerDisplayOnHover := Reg.ReadBool(STR_REG_SEC_VIEWER_ONHOVER)
            else SecondaryViewerDisplayOnHover := True;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_AUDIO)
            then SecondaryViewerAudio := Reg.ReadBool(STR_REG_SEC_VIEWER_AUDIO)
            else SecondaryViewerAudio := True;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_AUDIO_NO_STATUS)
            then SecondaryViewerAudioNoProgress := Reg.ReadBool(STR_REG_SEC_VIEWER_AUDIO_NO_STATUS)
            else SecondaryViewerAudioNoProgress := False;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_AUDIO_NO_INFO)
            then SecondaryViewerAudioNoInfo := Reg.ReadBool(STR_REG_SEC_VIEWER_AUDIO_NO_INFO)
            else SecondaryViewerAudioNoInfo := False;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_AUDIO_AA)
            then SecondaryViewerAudioAA := Reg.ReadInteger(STR_REG_SEC_VIEWER_AUDIO_AA)
            else SecondaryViewerAudioAA := 4;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_PICTURE)
            then SecondaryViewerPicture := Reg.ReadBool(STR_REG_SEC_VIEWER_PICTURE)
            else SecondaryViewerPicture := True;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_PICTURE_RESAMPLER)
            then SecondaryViewerPictureResampler := Reg.ReadInteger(STR_REG_SEC_VIEWER_PICTURE_RESAMPLER)
            else SecondaryViewerPictureResampler := 7;  //* 7 = Lanczos

        if Reg.ValueExists(STR_REG_SEC_VIEWER_HTML)
            then SecondaryViewerHTML := Reg.ReadBool(STR_REG_SEC_VIEWER_HTML)
            else SecondaryViewerHTML := True;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_TEXT_RTF)
            then SecondaryViewerTEXTRTF := Reg.ReadBool(STR_REG_SEC_VIEWER_TEXT_RTF)
            else SecondaryViewerTEXTRTF := True;

        if Reg.ValueExists(STR_REG_BALLOON_IN_LISTER)
            then BalloonInLister := Reg.ReadBool(STR_REG_BALLOON_IN_LISTER)
            else BalloonInLister := True;

        if Reg.ValueExists(STR_REG_BALLOON_IN_THUMBNAIL)
            then BalloonInThumbnailMode := Reg.ReadBool(STR_REG_BALLOON_IN_THUMBNAIL)
            else BalloonInThumbnailMode := True;

        if Reg.ValueExists(STR_REG_BALLOON_THUMBNAILS)
            then BalloonHasThumbs := Reg.ReadBool(STR_REG_BALLOON_THUMBNAILS)
            else BalloonHasThumbs := True;

        if Reg.ValueExists(STR_REG_BALLOON_HINT_TIMEOUT)
            then BalloonHintTimeOut := Reg.ReadInteger(STR_REG_BALLOON_HINT_TIMEOUT)
            else BalloonHintTimeOut := 1500;

        if Reg.ValueExists(STR_REG_TYPES_BROWSER)
            then TypesBrowser := Reg.ReadString(STR_REG_TYPES_BROWSER)
            else TypesBrowser := FILE_TYPES_BROWSER;

        if Reg.ValueExists(STR_REG_TYPES_AUDIO)
            then TypesAudio := Reg.ReadString(STR_REG_TYPES_AUDIO)
            else TypesAudio := FILE_TYPES_AUDIO;

        if Reg.ValueExists(STR_REG_TYPES_TEXT)
            then TypesText := Reg.ReadString(STR_REG_TYPES_TEXT)
            else TypesText := FILE_TYPES_TEXT;

        if Reg.ValueExists(STR_REG_TYPES_VIDEO)
            then TypesVideo := Reg.ReadString(STR_REG_TYPES_VIDEO)
            else TypesVideo := FILE_TYPES_VIDEO;

        if Reg.ValueExists(STR_REG_BALLOON_IN_LOCATIONS)
            then BalloonInLocations := Reg.ReadBool(STR_REG_BALLOON_IN_LOCATIONS)
            else BalloonInLocations := True;

        if Reg.ValueExists(STR_REG_URL_DECODE_FILENAMES)
            then AutoURLDecodeFileNames := Reg.ReadBool(STR_REG_URL_DECODE_FILENAMES)
            else AutoURLDecodeFileNames := True;

        if Reg.ValueExists(STR_REG_UPPERCASE_FIRST_LETTER_FOR_FILENAMES)
            then AutoConvertFirstLetterInFileNames := Reg.ReadBool(STR_REG_UPPERCASE_FIRST_LETTER_FOR_FILENAMES)
            else AutoConvertFirstLetterInFileNames := False;

        if Reg.ValueExists(STR_REG_UNDERSCORES_2_SPACE_FOR_FILENAMES)
            then AutoConvertUnderscoresInFileNames := Reg.ReadBool(STR_REG_UNDERSCORES_2_SPACE_FOR_FILENAMES)
            else AutoConvertUnderscoresInFileNames := False;

        if Reg.ValueExists(STR_REG_RECENT_FILES_COUNT)
            then RecentFileCount := Reg.ReadInteger(STR_REG_RECENT_FILES_COUNT)
            else RecentFileCount := 10;

        if Reg.ValueExists(STR_REG_DONTDISAPLAY_TFDDB)
            then DontDisplayTFDDB := Reg.ReadBool(STR_REG_DONTDISAPLAY_TFDDB)
            else DontDisplayTFDDB := True;

        if Reg.ValueExists(STR_REG_SHOW_HIDDENSYSTEM)
            then ShowHiddenSystem := Reg.ReadBool(STR_REG_SHOW_HIDDENSYSTEM)
            else ShowHiddenSystem := True;

        if Reg.ValueExists(STR_REG_FILENAME_TEMPLATE)
            then FilenameTemplate := Reg.ReadString(STR_REG_FILENAME_TEMPLATE)
            else FilenameTemplate := '{n}{e}';

        if Reg.ValueExists(STR_REG_RECENT_FILES_DEL_INVALID)
            then AutoDelInvalidRecentLinks := Reg.ReadBool(STR_REG_RECENT_FILES_DEL_INVALID)
            else AutoDelInvalidRecentLinks := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_AUDIOFILES_APIC)
            then ThumbnailsForAudioFilesAsAPIC := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_APIC)
            else ThumbnailsForAudioFilesAsAPIC := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_APIC)
            then ThumbnailsForAudioFilesPreferCoverArts := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_APIC)
            else ThumbnailsForAudioFilesPreferCoverArts := True;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_SAMPLES)
            then ThumbnailsForAudioFilesPreferSamples := Reg.ReadBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_SAMPLES)
            else ThumbnailsForAudioFilesPreferSamples := False;

        if Reg.ValueExists(STR_REG_THUMBNAILER_TEXT_ZOOM)
            then ThumbnailerTextZoom := Reg.ReadInteger(STR_REG_THUMBNAILER_TEXT_ZOOM)
            else ThumbnailerTextZoom := 100;

        if Reg.ValueExists(STR_REG_SEC_VIEWER_HTML_UNKNOWN_TYPES)
            then SecondaryViewerHTMLAllFiles := Reg.ReadBool(STR_REG_SEC_VIEWER_HTML_UNKNOWN_TYPES)
            else SecondaryViewerHTMLAllFiles := False;

        if Reg.ValueExists(STR_REG_NO_CDN_TYPECHANGE)
            then NoCDN_TYPECHANGE := Reg.ReadBool(STR_REG_NO_CDN_TYPECHANGE)
            else NoCDN_TYPECHANGE := False;

        if Reg.ValueExists(STR_REG_TRY_WEBBROWSER)
            then TryWebBrowserForUnknownFileTypes := Reg.ReadBool(STR_REG_TRY_WEBBROWSER)
            else TryWebBrowserForUnknownFileTypes := False;

        if Reg.ValueExists(STR_REG_LISTER_HAS_THE_FOCUS)
            then ListerHasTheFocus := Reg.ReadBool(STR_REG_LISTER_HAS_THE_FOCUS)
            else ListerHasTheFocus := False;

        if Reg.ValueExists(STR_REG_BALLOON_NO_FILENAME_FOCUS)
            then NoBalloonWhenFileNameHasFocus := Reg.ReadBool(STR_REG_BALLOON_NO_FILENAME_FOCUS)
            else NoBalloonWhenFileNameHasFocus := False;

        if Reg.ValueExists(STR_REG_LISTER_USE_CUSTOM_FONT_COLOR)
            then UseCustomFontColor := Reg.ReadBool(STR_REG_LISTER_USE_CUSTOM_FONT_COLOR)
            else UseCustomFontColor := False;

        if Reg.ValueExists(STR_REG_LISTER_CUSTOM_FONT_COLOR)
            then CustomFontColor := Reg.ReadInteger(STR_REG_LISTER_CUSTOM_FONT_COLOR)
            else CustomFontColor := $0;

        if Reg.ValueExists(STR_REG_LISTER_USE_CUSTOM_BACKGROUND_COLOR)
            then UseCustomBackgroundColor := Reg.ReadBool(STR_REG_LISTER_USE_CUSTOM_BACKGROUND_COLOR)
            else UseCustomBackgroundColor := False;

        if Reg.ValueExists(STR_REG_LISTER_CUSTOM_BACKGROUND_COLOR)
            then CustomBackgroundColor := Reg.ReadInteger(STR_REG_LISTER_CUSTOM_BACKGROUND_COLOR)
            else CustomBackgroundColor := Color32(clWindow);

        if Reg.ValueExists(STR_REG_DONT_ASK_IF_NO_FILE_TYPE_SPECIFIED)
            then DontAskIfNoFileTypeSpecified := Reg.ReadBool(STR_REG_DONT_ASK_IF_NO_FILE_TYPE_SPECIFIED)
            else DontAskIfNoFileTypeSpecified := False;

        if Reg.ValueExists(STR_REG_THUMBNAILS_FOLDER_WIN_ICON)
            then ThumbnailsUseWindowsFolderIcon := Reg.ReadBool(STR_REG_THUMBNAILS_FOLDER_WIN_ICON)
            else ThumbnailsUseWindowsFolderIcon := True;

        if Reg.ValueExists(STR_REG_UI_FONT_SIZE)
            then UIFontSize := Reg.ReadInteger(STR_REG_UI_FONT_SIZE)
            else UIFontSize := 10;

        if Reg.ValueExists(STR_REG_UI_STYLE)
            then UIStyle := Reg.ReadString(STR_REG_UI_STYLE)
            else UIStyle := 'Windows';

        if Reg.ValueExists(STR_REG_UI_STYLE_WINDOW_OVERRIDE)
            then UIStyleWindowOverride := Reg.ReadBool(STR_REG_UI_STYLE_WINDOW_OVERRIDE)
            else UIStyleWindowOverride := False;

        if Reg.ValueExists(STR_REG_UI_STYLE_INTEGRATED_OVERRIDE)
            then UIStyleIntegratedOverride := Reg.ReadBool(STR_REG_UI_STYLE_INTEGRATED_OVERRIDE)
            else UIStyleIntegratedOverride := False;

        if Reg.ValueExists('SkeezAnimationIcon')
            then SkeezAnimationIcon := Reg.ReadInteger('SkeezAnimationIcon');
        if Reg.ValueExists('SkeezAnimationIconX')
            then SkeezAnimationIconX := Reg.ReadInteger('SkeezAnimationIconX');
        if Reg.ValueExists('SkeezAnimationIconY')
            then SkeezAnimationIconY := Reg.ReadInteger('SkeezAnimationIconY');
        if Reg.ValueExists('SkeezAnimationIconDuration')
            then SkeezAnimationIconDuration := Reg.ReadInteger('SkeezAnimationIconDuration');

        if Reg.ValueExists('SkeezAnimationFileName')
            then SkeezAnimationFileName := Reg.ReadInteger('SkeezAnimationFileName');
        if Reg.ValueExists('SkeezAnimationFileNameX')
            then SkeezAnimationFileNameX := Reg.ReadInteger('SkeezAnimationFileNameX');
        if Reg.ValueExists('SkeezAnimationFileNameY')
            then SkeezAnimationFileNameY := Reg.ReadInteger('SkeezAnimationFileNameY');
        if Reg.ValueExists('SkeezAnimationFileNameDuration')
            then SkeezAnimationFileNameDuration := Reg.ReadInteger('SkeezAnimationFileNameDuration');

        if Reg.ValueExists('SkeezAnimationFileSize')
            then SkeezAnimationFileSize := Reg.ReadInteger('SkeezAnimationFileSize');
        if Reg.ValueExists('SkeezAnimationFileSizeX')
            then SkeezAnimationFileSizeX := Reg.ReadInteger('SkeezAnimationFileSizeX');
        if Reg.ValueExists('SkeezAnimationFileSizeY')
            then SkeezAnimationFileSizeY := Reg.ReadInteger('SkeezAnimationFileSizeY');
        if Reg.ValueExists('SkeezAnimationFileSizeDuration')
            then SkeezAnimationFileSizeDuration := Reg.ReadInteger('SkeezAnimationFileSizeDuration');

        if Reg.ValueExists('SkeezAnimationDateTime')
            then SkeezAnimationDateTime := Reg.ReadInteger('SkeezAnimationDateTime');
        if Reg.ValueExists('SkeezAnimationDateTimeX')
            then SkeezAnimationDateTimeX := Reg.ReadInteger('SkeezAnimationDateTimeX');
        if Reg.ValueExists('SkeezAnimationDateTimeY')
            then SkeezAnimationDateTimeY := Reg.ReadInteger('SkeezAnimationDateTimeY');
        if Reg.ValueExists('SkeezAnimationDateTimeDuration')
            then SkeezAnimationDateTimeDuration := Reg.ReadInteger('SkeezAnimationDateTimeDuration');

        if Reg.ValueExists('SkeezAnimationDescription')
            then SkeezAnimationDescription := Reg.ReadInteger('SkeezAnimationDescription');
        if Reg.ValueExists('SkeezAnimationDescriptionX')
            then SkeezAnimationDescriptionX := Reg.ReadInteger('SkeezAnimationDescriptionX');
        if Reg.ValueExists('SkeezAnimationDescriptionY')
            then SkeezAnimationDescriptionY := Reg.ReadInteger('SkeezAnimationDescriptionY');
        if Reg.ValueExists('SkeezAnimationDescriptionDuration')
            then SkeezAnimationDescriptionDuration := Reg.ReadInteger('SkeezAnimationDescriptionDuration');

        if Reg.ValueExists('SkeezAnimationDetails')
            then SkeezAnimationDetails := Reg.ReadInteger('SkeezAnimationDetails');
        if Reg.ValueExists('SkeezAnimationDetailsX')
            then SkeezAnimationDetailsX := Reg.ReadInteger('SkeezAnimationDetailsX');
        if Reg.ValueExists('SkeezAnimationDetailsY')
            then SkeezAnimationDetailsY := Reg.ReadInteger('SkeezAnimationDetailsY');
        if Reg.ValueExists('SkeezAnimationDetailsDuration')
            then SkeezAnimationDetailsDuration := Reg.ReadInteger('SkeezAnimationDetailsDuration');

        Reg.CloseKey;

        Reg.OpenKey(STR_REG_FD_SETTINGS + '\', True);

        //* Read only values
        if Reg.ValueExists(STR_REG_BASS_DONTINIT)
            then BASSDontInit := Reg.ReadBool(STR_REG_BASS_DONTINIT)
            else Reg.WriteBool(STR_REG_BASS_DONTINIT, False);

        Reg.CloseKey;

        //* Font
        Font.Name := 'Tahoma';
        Font.Size := 10;

        if Reg.KeyExists(STR_REG_FD_SETTINGS + '\' + STR_REG_FD_FONT_SETTINGS + '\') then begin
            Reg.OpenKey(STR_REG_FD_SETTINGS + '\' + STR_REG_FD_FONT_SETTINGS + '\', False);
            if Reg.ValueExists('Name')
                then Font.Name := Reg.ReadString('Name')
                //else Font.Name := 'MS Sans Serif';
                else Font.Name := 'Tahoma';
            if Reg.ValueExists('Size')
                then Font.Size := Reg.ReadInteger('Size')
                //else Font.Size := 8;
                else Font.Size := 10;
            if Reg.ValueExists('Italic')
                then if Reg.ReadBool('Italic')
                    then Font.Style := Font.Style + [fsItalic]
                    else Font.Style := Font.Style - [fsItalic];
            if Reg.ValueExists('Bold')
                then if Reg.ReadBool('Bold')
                    then Font.Style := Font.Style + [fsBold]
                    else Font.Style := Font.Style - [fsBold];
        end;

    finally
        Reg.CloseKey;
        Reg.Free;
    end;
end;

procedure TFileSystemDialogsSettings.SaveSettings(AnimationOptionsToo: Boolean);
var
    Reg: TRegistry;
begin
    if RuningOn64BitOS then begin
        Reg := TRegistry.Create(KEY_WOW64_64KEY OR KEY_ALL_ACCESS);
    end else begin
        Reg := TRegistry.Create;
    end;
    try
        Reg.RootKey := HKEY_CURRENT_USER;

        //* Global
        Reg.OpenKey(STR_REG_FD_SETTINGS + '\', True);
        Reg.WriteBool(STR_REG_USE_LAST_FOLDER, UseLastFolder);
        Reg.WriteBool(STR_REG_SINGLE_SETUP, SingleSetup);
        Reg.WriteBool(STR_REG_FILE_DATETIME_PRECISION, FileDateTimeMSPrecision);
        Reg.WriteBool(STR_REG_READ_CREATED_FOLDER, AutoGoIntoCreatedFolder);
        Reg.WriteBool(STR_REG_SAVE_SETTINGS_ON_CANCEL, SaveSettingsOnCancel);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_DIRS, ThumbnailsForDirs);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_DIR_AUDIO, ThumbnailsForDirsAudio);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES, ThumbnailsForAudioFiles);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_SHADOW, ThumbnailsForAudioFilesShadow);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_TEXTFILES, ThumbnailsForTextFiles);
        Reg.WriteBool(STR_REG_THUMBNAILS_GIF_ANIM, ThumbnailsGIFAnim);
        Reg.WriteBool(STR_REG_THUMBNAILS_ZOOM_X2, ThumbnailsSmallZoomx2);
        Reg.WriteInteger(STR_REG_THUMBNAILS_WIDTH, ThumbnailsWidth);
        Reg.WriteInteger(STR_REG_THUMBNAILS_HEIGHT, ThumbnailsHeight);
        Reg.WriteInteger(STR_REG_THUMBNAILS_BALLOON_WIDTH, ThumbnailsBalloonWidth);
        Reg.WriteInteger(STR_REG_THUMBNAILS_BALLOON_HEIGHT, ThumbnailsBalloonHeight);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SKEEZ_WIDTH, ThumbnailsSkeezWidth);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SKEEZ_HEIGHT, ThumbnailsSkeezHeight);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SIZE_PRESET, ThumbnailsSizePreset);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_L, ThumbnailsSampleColorL);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_R, ThumbnailsSampleColorR);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_B, ThumbnailsSampleColorB);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_L_VIEWER, ThumbnailsSampleColorLViewer);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_R_VIEWER, ThumbnailsSampleColorRViewer);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_COLOR_B_VIEWER, ThumbnailsSampleColorBViewer);
        Reg.WriteInteger(STR_REG_THUMBNAILS_SAMPLE_AA, ThumbnailsSampleAALEvel);
        Reg.WriteInteger(STR_REG_THUMBNAILS_FOLDER_COLOR, ThumbnailsFolderIconColor);
        Reg.WriteBool(STR_REG_THUMBNAILS_ALL_THREADS, ThumbnailsThreadsAll);
        Reg.WriteBool(STR_REG_SAVE_SESC, SaveSESC);
        Reg.WriteBool(STR_REG_FS_FORCE_EXTCASE, ForceExtCase);
        Reg.WriteBool(STR_REG_FS_FORCE_EXTCASEU, ForceExtCaseUpper);
        Reg.WriteBool(STR_REG_FS_FORCE_EXTCASEL, ForceExtCaseLower);
        Reg.WriteBool(STR_REG_INACTIVE_TREEVIEW, InactiveTreeViewUntilFirstTouch);
        Reg.WriteBool(STR_REG_OMIT_FILE_EXT, OmitFileExt);
        Reg.WriteBool(STR_REG_DETAILS, DisplayDetails);
        Reg.WriteInteger(STR_REG_SETTINGS_TOP, SettingsWindowPosition.Y);
        Reg.WriteInteger(STR_REG_SETTINGS_LEFT, SettingsWindowPosition.X);
        Reg.WriteInteger(STR_REG_SETTINGS_PAGE_INDEX, SettingsPageIndex);
        Reg.WriteInteger(STR_REG_SETTINGS_PAGE_THUMBS_INDEX, SettingsPageThumbsIndex);
        Reg.WriteInteger(STR_REG_SETTINGS_PAGE_PREFS_INDEX, SettingsPagePrefsIndex);
        Reg.WriteInteger(STR_REG_RECENT_FOLDER_COUNT, RecentFolderCount);
        Reg.WriteBool(STR_REG_SHOW_SCROLLBAR_RECENT, ShowScrollbarsRecent);
        Reg.WriteBool(STR_REG_SHOW_SCROLLBAR_TREEVIEW, ShowScrollbarsTreeView);
        Reg.WriteBool(STR_REG_THUMBNAILS_CACHED_GRAPHICAL, ThumbnailsCachePicture);
        Reg.WriteBool(STR_REG_THUMBNAILS_CACHED_AUDIO, ThumbnailsCacheAudio);
        Reg.WriteBool(STR_REG_SETTINGS_DISPLAY_EXTS_CONVERTED, ConvertDisplayExtensions);
        Reg.WriteBool(STR_REG_SEC_VIEWER, UseSecondaryViewer);
        Reg.WriteBool(STR_REG_SEC_VIEWER_SPECIFY, SecondaryViewerSpecified);
        Reg.WriteInteger(STR_REG_SEC_VIEWER_MONITOR, SecondaryViewerMonitorNo);
        Reg.WriteBool(STR_REG_SEC_VIEWER_ONHOVER, SecondaryViewerDisplayOnHover);
        Reg.WriteBool(STR_REG_SEC_VIEWER_AUDIO, SecondaryViewerAudio);
        Reg.WriteBool(STR_REG_SEC_VIEWER_AUDIO_NO_STATUS, SecondaryViewerAudioNoProgress);
        Reg.WriteBool(STR_REG_SEC_VIEWER_AUDIO_NO_INFO, SecondaryViewerAudioNoInfo);
        Reg.WriteInteger(STR_REG_SEC_VIEWER_AUDIO_AA, SecondaryViewerAudioAA);
        Reg.WriteBool(STR_REG_SEC_VIEWER_PICTURE, SecondaryViewerPicture);
        Reg.WriteInteger(STR_REG_SEC_VIEWER_PICTURE_RESAMPLER, SecondaryViewerPictureResampler);
        Reg.WriteBool(STR_REG_SEC_VIEWER_HTML, SecondaryViewerHTML);
        Reg.WriteBool(STR_REG_SEC_VIEWER_TEXT_RTF, SecondaryViewerTEXTRTF);
        Reg.WriteBool(STR_REG_BALLOON_IN_LISTER, BalloonInLister);
        Reg.WriteBool(STR_REG_BALLOON_IN_THUMBNAIL, BalloonInThumbnailMode);
        Reg.WriteBool(STR_REG_BALLOON_THUMBNAILS, BalloonHasThumbs);
        Reg.WriteInteger(STR_REG_BALLOON_HINT_TIMEOUT, BalloonHintTimeOut);
        Reg.WriteString(STR_REG_TYPES_BROWSER, TypesBrowser);
        Reg.WriteString(STR_REG_TYPES_AUDIO, TypesAudio);
        Reg.WriteString(STR_REG_TYPES_TEXT, TypesText);
        Reg.WriteString(STR_REG_TYPES_VIDEO, TypesVideo);
        Reg.WriteBool(STR_REG_BALLOON_IN_LOCATIONS, BalloonInLocations);
        Reg.WriteBool(STR_REG_URL_DECODE_FILENAMES, AutoURLDecodeFileNames);
        Reg.WriteBool(STR_REG_UPPERCASE_FIRST_LETTER_FOR_FILENAMES, AutoConvertFirstLetterInFileNames);
        Reg.WriteBool(STR_REG_UNDERSCORES_2_SPACE_FOR_FILENAMES, AutoConvertUnderscoresInFileNames);
        Reg.WriteInteger(STR_REG_RECENT_FILES_COUNT, RecentFileCount);
        Reg.WriteBool(STR_REG_DONTDISAPLAY_TFDDB, DontDisplayTFDDB);
        Reg.WriteBool(STR_REG_SHOW_HIDDENSYSTEM, ShowHiddenSystem);
        Reg.WriteString(STR_REG_FILENAME_TEMPLATE, FilenameTemplate);
        Reg.WriteBool(STR_REG_RECENT_FILES_DEL_INVALID, AutoDelInvalidRecentLinks);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_APIC, ThumbnailsForAudioFilesAsAPIC);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_APIC, ThumbnailsForAudioFilesPreferCoverArts);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOR_AUDIOFILES_PREFER_SAMPLES, ThumbnailsForAudioFilesPreferSamples);
        Reg.WriteInteger(STR_REG_THUMBNAILER_TEXT_ZOOM, ThumbnailerTextZoom);
        Reg.WriteBool(STR_REG_SEC_VIEWER_HTML_UNKNOWN_TYPES, SecondaryViewerHTMLAllFiles);
        Reg.WriteBool(STR_REG_NO_CDN_TYPECHANGE, NoCDN_TYPECHANGE);
        Reg.WriteBool(STR_REG_TRY_WEBBROWSER, TryWebBrowserForUnknownFileTypes);
        Reg.WriteBool(STR_REG_LISTER_HAS_THE_FOCUS, ListerHasTheFocus);
        Reg.WriteBool(STR_REG_BALLOON_NO_FILENAME_FOCUS, NoBalloonWhenFileNameHasFocus);
        Reg.WriteBool(STR_REG_LISTER_USE_CUSTOM_FONT_COLOR, UseCustomFontColor);
        Reg.WriteInteger(STR_REG_LISTER_CUSTOM_FONT_COLOR, CustomFontColor);
        Reg.WriteBool(STR_REG_LISTER_USE_CUSTOM_BACKGROUND_COLOR, UseCustomBackgroundColor);
        Reg.WriteInteger(STR_REG_LISTER_CUSTOM_BACKGROUND_COLOR, CustomBackgroundColor);
        Reg.WriteBool(STR_REG_DONT_ASK_IF_NO_FILE_TYPE_SPECIFIED, DontAskIfNoFileTypeSpecified);
        Reg.WriteBool(STR_REG_THUMBNAILS_FOLDER_WIN_ICON, ThumbnailsUseWindowsFolderIcon);
        Reg.WriteInteger(STR_REG_UI_FONT_SIZE, UIFontSize);
        Reg.WriteString(STR_REG_UI_STYLE, UIStyle);
        Reg.WriteBool(STR_REG_UI_STYLE_WINDOW_OVERRIDE, UIStyleWindowOverride);
        Reg.WriteBool(STR_REG_UI_STYLE_INTEGRATED_OVERRIDE, UIStyleIntegratedOverride);

        if AnimationOptionsToo then begin

            Reg.WriteInteger('SkeezAnimationIcon', SkeezAnimationIcon);
            Reg.WriteInteger('SkeezAnimationIconX', SkeezAnimationIconX);
            Reg.WriteInteger('SkeezAnimationIconY', SkeezAnimationIconY);
            Reg.WriteInteger('SkeezAnimationIconDuration', SkeezAnimationIconDuration);

            Reg.WriteInteger('SkeezAnimationFileName', SkeezAnimationFileName);
            Reg.WriteInteger('SkeezAnimationFileNameX', SkeezAnimationFileNameX);
            Reg.WriteInteger('SkeezAnimationFileNameY', SkeezAnimationFileNameY);
            Reg.WriteInteger('SkeezAnimationFileNameDuration', SkeezAnimationFileNameDuration);

            Reg.WriteInteger('SkeezAnimationFileSize', SkeezAnimationFileSize);
            Reg.WriteInteger('SkeezAnimationFileSizeX', SkeezAnimationFileSizeX);
            Reg.WriteInteger('SkeezAnimationFileSizeY', SkeezAnimationFileSizeY);
            Reg.WriteInteger('SkeezAnimationFileSizeDuration', SkeezAnimationFileSizeDuration);

            Reg.WriteInteger('SkeezAnimationDateTime', SkeezAnimationDateTime);
            Reg.WriteInteger('SkeezAnimationDateTimeX', SkeezAnimationDateTimeX);
            Reg.WriteInteger('SkeezAnimationDateTimeY', SkeezAnimationDateTimeY);
            Reg.WriteInteger('SkeezAnimationDateTimeDuration', SkeezAnimationDateTimeDuration);

            Reg.WriteInteger('SkeezAnimationDescription', SkeezAnimationDescription);
            Reg.WriteInteger('SkeezAnimationDescriptionX', SkeezAnimationDescriptionX);
            Reg.WriteInteger('SkeezAnimationDescriptionY', SkeezAnimationDescriptionY);
            Reg.WriteInteger('SkeezAnimationDescriptionDuration', SkeezAnimationDescriptionDuration);

            Reg.WriteInteger('SkeezAnimationDetails', SkeezAnimationDetails);
            Reg.WriteInteger('SkeezAnimationDetailsX', SkeezAnimationDetailsX);
            Reg.WriteInteger('SkeezAnimationDetailsY', SkeezAnimationDetailsY);
            Reg.WriteInteger('SkeezAnimationDetailsDuration', SkeezAnimationDetailsDuration);

        end;

        Reg.CloseKey;

        //* Font
        Reg.OpenKey(STR_REG_FD_SETTINGS + '\' + STR_REG_FD_FONT_SETTINGS + '\', True);
        Reg.WriteString('Name', Font.Name);
        Reg.WriteInteger('Size', Font.Size);
        Reg.WriteBool('Italic', fsItalic in Font.Style);
        Reg.WriteBool('Bold', fsBold in Font.Style);

    finally
        Reg.CloseKey;
        Reg.Free;
    end;
end;

function TFDStyle2String(FDStyle: TFDStyle): String;
begin
    Result := '';
    case FDStyle of
        fdsWindows: Result := 'Windows';
        fdsAmakrits: Result := 'Amakrits';
        fdsAmethystKamri: Result := 'Amethyst Kamri';
        fdsAquaGraphite: Result := 'Aqua Graphite';
        fdsAquaLightSlate: Result := 'Aqua Light Slate';
        fdsAuric: Result := 'Auric';
        fdsCarbon: Result := 'Carbon';
        fdsCharcoalDarkSlate: Result := 'Charcoal Dark Slate';
        fdsCobaltXEMedia: Result := 'Cobalt XEMedia';
        fdsCyanDusk: Result := 'Cyan Dusk';
        fdsCyanNight: Result := 'Cyan Night';
        fdsEmeraldLightSlate: Result := 'Emerald Light Slate';
        fdsGlossy: Result := 'Glossy';
        fdsGlow: Result := 'Glow';
        fdsGoldenGraphite: Result := 'Golden Graphite';
        fdsIcebergClassico: Result := 'Iceberg Classico';
        fdsLavenderClassico: Result := 'Lavender Classico';
        fdsLight: Result := 'Light';
        fdsLuna: Result := 'Luna';
        fdsObsidian: Result := 'Obsidian';
        fdsOnyxBlue: Result := 'Onyx Blue';
        fdsRubyGraphite: Result := 'Ruby Graphite';
        fdsSapphireKamri: Result := 'Sapphire Kamri';
        fdsSilver: Result := 'Silver';
        fdsSky: Result := 'Sky';
        fdsSlateClassico: Result := 'Slate Classico';
        fdsSmokeyQuartzKamri: Result := 'Smokey Quartz Kamri';
        fdsTabletDark: Result := 'Tablet Dark';
        fdsTabletLight: Result := 'Tablet Light';
        fdsTurquoiseGray: Result := 'Turquoise Gray';
        fdsWindows10: Result := 'Windows10';
        fdsWindows10Blue: Result := 'Windows10 Blue';
        fdsWindows10Dark: Result := 'Windows10 Dark';
        fdsWindows10Green: Result := 'Windows10 Green';
        fdsWindows10Purple: Result := 'Windows10 Purple';
        fdsWindows10SlateGray: Result := 'Windows10 SlateGray';
    end;
end;

function LoadFilesystemDialogsLibrary: Boolean;
begin
    Result := False;
    FilesystemDialogsDLLHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));
    if FilesystemDialogsDLLHandle <> 0 then begin
        FilesystemDialogsOpenDialog := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_OpenDialog));
        FilesystemDialogsSaveDialog := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_SaveDialog));
        FilesystemDialogsFolderDialog := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FolderDialog));
        FilesystemDialogsSettings := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_Settings));
        FilesystemDialogsFreeDialogResults := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FreeDialogResults));
        if (@FilesystemDialogsOpenDialog = nil)
        OR (@FilesystemDialogsSaveDialog = nil)
        OR (@FilesystemDialogsFolderDialog = nil)
        OR (@FilesystemDialogsSettings = nil)
        OR (@FilesystemDialogsFreeDialogResults = nil)
            then Result := False
            else Result := True;
        if NOT Result then begin
            FreeLibrary(FilesystemDialogsDLLHandle);
            FilesystemDialogsDLLHandle := 0;
        end;
    end;
end;

function FreeFilesystemDialogsLibrary: Boolean;
begin
    FreeLibrary(FilesystemDialogsDLLHandle);
    FilesystemDialogsDLLHandle := 0;
    Result := True;
end;

Initialization

    {$IFNDEF FilesystemDialogs_Native_Mode}
    LoadFilesystemDialogsLibrary;
    {$ENDIF}

Finalization

    {$IFNDEF FilesystemDialogs_Native_Mode}
    FreeFilesystemDialogsLibrary;
    {$ENDIF}

end.




