//********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs Library 2.1.3.6840  3delite 2006-2024                                                                *
//*     See Filesystem Dialogs Library ReadMe.txt for details.                                                                   *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* 	http://www.shareit.com/product.html?productid=300958160                                                                  *
//* Shareware License: 95                                                                                                       *
//* 	http://www.shareit.com/product.html?productid=300070165                                                                  *
//* Commercial License: 450                                                                                                     *
//* 	http://www.shareit.com/product.html?productid=300070166                                                                  *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                          *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit FilesystemDialogsDynamic;

interface

Uses
    Windows,
    ShlObj;

const
    FILENAME_DLL_FD = 'FilesystemDialogs.dll';
    //* Exported function names
    NAME_FD_OpenDialog          = 'FilesystemDialogs_OpenDialog';
    NAME_FD_SaveDialog          = 'FilesystemDialogs_SaveDialog';
    NAME_FD_FolderDialog        = 'FilesystemDialogs_FolderDialog';
    NAME_FD_Settings            = 'FilesystemDialogs_SettingsWindow';
    NAME_FD_FreeDialogResults   = 'FilesystemDialogs_FreeDialogResults';

const
    //* Internal max. path size 7800 bytes (10)
    FD_MAX_PATH = MAX_PATH * 10;

const
    {$EXTERNALSYM WM_USER}
    WM_USER = $0400;

//* Filesystem Dialogs messages
const

    FDM_NAME_SETTINGS_CHANGED = 'Filesystem Dialogs Settings Changed';

    //* Pointer to TOpenDialogResults is in wParam
    //* Pointer to TOpenDialogParams is in lParam
    //* You get the messages in the hookproc and to the specified Parent window handle
    FDM_BASE = 2 * WM_USER + 1;

    //* Filesystem Dialogs results message
    FDM_RESULTS = FDM_BASE + 1;

    //* Load settings, WPARAM = 0, LPARAM = 0
    FDM_LOADSETTINGS = FDM_BASE + 2;

    //* Free memory that you have allocated for the dialog('s params, results)
    FDM_FREEMEM = FDM_BASE + 3;

    //* The dialog form is closing
    FDM_CLOSING = FDM_BASE + 4;

    //* The dialog form's window handle in wParam, pointer to TOpenDialogParams is in lParam
    FDM_WINDOW_HANDLE = FDM_BASE + 5;

    //* Realing the integrated form to the parent
    FDM_REALIGN_INTEGRATED_WINDOW = FDM_BASE + 6;

    //* Sent on file name selection, file name is in wParam as PChar (unicode), item's PIDL in lParam
    FDM_ON_SELECT_FILE_NAME = FDM_BASE + 7;

    //* Refresh folder
    FDM_REFRESH_FOLDER = FDM_BASE + 8;

    //* Used internally
    FDM_THUMBNAIL_READY = FDM_BASE + 9;

    //* Used internally
    FDM_ADD_THUMBNAIL_FILENAME = FDM_BASE + 10;

    //* Used internally
    FDM_THUMBNAILER_THREAD_FINISHED = FDM_BASE + 11;

    //* Used internally
    FDM_THUMBNAILER_CONTROL_THREAD_TERMINATE = FDM_BASE + 12;

    //* Used internally
    FDM_SECONDARYVIEWER_PICTURE_READY = FDM_BASE + 13;

    //* Used internally
    FDM_SECONDARYVIEWER_AUDIOPICTURE_READY = FDM_BASE + 14;

    //* Used internally
    FDM_VIEWER_AUDIOPICTURE_READY = FDM_BASE + 15;

    //* Custom previewer query file, wParam = PChar file name, lParam = PIDL
    FDM_PREVIEWER_CUSTOM_QUERY = FDM_BASE + 16;

    //* Custom previewer open previewer, Wnd = handle for the panel, wParam = PChar file name, lParam = PIDL
    FDM_PREVIEWER_CUSTOM_DISPLAY = FDM_BASE + 17;

    //* Custom previewer close
    FDM_PREVIEWER_CUSTOM_CLOSE = FDM_BASE + 18;

    //* Custom previewer resized
    FDM_PREVIEWER_CUSTOM_RESIZED = FDM_BASE + 19;

    //* Used internally
    FDM_VIEWER_AUDIOPICTURE_PROGRESS = FDM_BASE + 20;

    //* Used internally
    FDM_SECONDARYVIEWER_VIEWER_AUDIOPICTURE_PROGRESS = FDM_BASE + 21;

    //* Navigate to folder specified as PIDL in WPARAM, LPARAM = 0
    FDM_NAVIGATE_TO_PIDL = FDM_BASE + 22;

    //* Trigger sending of selected files and folders for hookproc, WPARAM = 0, LPARAM = 0
    FDM_QUERY_SELECTED_ITEMS = FDM_BASE + 23;

    //* Abort (stop) listing a folder, WPARAM = 0, LPARAM = 0
    FDM_CANCEL_LISTING_ITEMS = FDM_BASE + 24;

    //* Used internally
    FDM_VIEWER_GIFANIM_READY = FDM_BASE + 25;

    //* Used internally
    FDM_SECONDARYVIEWER_GIFANIM_READY = FDM_BASE + 26;

    //* Request the current folder, returns POpenDialogOnCurrentFolder in WPARAM, POpenDialogParams in LPARAM
    FDM_GET_CURRENT_FOLDER = FDM_BASE + 27;

    //* A dialog window is activated, returns 0 in WPARAM, POpenDialogParams in LPARAM
    FDM_WINDOW_ACTIVATED = FDM_BASE + 28;

    //* Activate a dialog window, WPARAM = 0, LPARAM = 0
    FDM_WINDOW_ACTIVATE = FDM_BASE + 29;

    //* Select all files and folders, WPARAM = 0, LPARAM = 0
    FDM_SELECT_ALL = FDM_BASE + 30;

    //* Invert selected and unselected files and folders, WPARAM = 0, LPARAM = 0
    FDM_INVERT_SELECTION = FDM_BASE + 31;

    //* Invert selected and unselected files and folders, WPARAM = 0, LPARAM = POpenDialogSelectPattern
    FDM_SELECT_PATTERN = FDM_BASE + 32;

    //* Current folder changed, WPARAM = PIDL, LPARAM = POpenDialogParams
    FDM_CURRENT_FOLDER_CHANGED = FDM_BASE + 33;

    //* Editing text, WPARAM = 1 when started, 0 when exited, LPARAM = POpenDialogSelectPattern
    FDM_IN_EDIT = FDM_BASE + 34;

    //* Deselect all files and folders, WPARAM = 0, LPARAM = 0
    FDM_SELECT_NONE = FDM_BASE + 35;

    //* Used internally
    FDM_FOLDER_SIZE_RESULT = FDM_BASE + 36;

    //* Get folder sizes, WPARAM = 0, LPARAM = 0
    FDM_COUNT_FOLDER_SIZES = FDM_BASE + 37;

    //* Display the search panel, WPARAM = 0, LPARAM = 0
    FDM_SHOW_SEARCH_PANEL = FDM_BASE + 38;

    //* Add a PIDL, WPARAM = PIDL, LPARAM = 0
    FDM_ADD_PIDL_TO_LIST = FDM_BASE + 39;

    //* Search completed, WPARAM = 0, LPARAM = 0
    FDM_SEARCH_COMPLETED = FDM_BASE + 40;

    //* Search completed, WPARAM = TFDSearchProgress, LPARAM = 0
    FDM_SEARCH_PROGRESS = FDM_BASE + 41;

    //* Style changed, WPARAM = TFDStyleChanged, LPARAM = 0
    FDM_STYLE_CHANGED = FDM_BASE + 42;

    //* Host, parent window, handle changed, WPARAM = NewHandle, LPARAM = 0
    FDM_HOST_HANDLE_CHANGED = FDM_BASE + 43;

    //* Reference to dialog class instance, WPARAM = TFormFileDialog, LPARAM = POpenDialogParams
    FDM_DIALOG_INSTANCE = FDM_BASE + 44;

    //* Pop-up settings menu, WPARAM = X, LPARAM = Y
    FDM_DIALOG_POPUP_SETTINGS_MENU = FDM_BASE + 45;

//* Many of the following flags are not supported (legacy Windows flags), if any of the missing flags are needed, please contact me.
type
    TOpenOption = (ofReadOnly, ofOverwritePrompt, ofHideReadOnly, ofNoChangeDir, ofShowHelp, ofNoValidate, ofAllowMultiSelect, ofExtensionDifferent, ofPathMustExist, ofFileMustExist, ofCreatePrompt, ofShareAware,
        ofNoReadOnlyReturn, ofNoTestFileCreate, ofNoNetworkButton, ofNoDereferenceLinks, ofEnableIncludeNotify, ofDontAddToRecent, ofForceShowHidden,
        ofNotModal,             //* Not modal dialogs - user can access the app.'s UI
        ofNoClose,              //* Dialog will not close after double clicking or clicking on 'Open/Save' button - usefull with the 'ofNotModal' flag
        ofIntegrate,            //* The dialog will appear in the control specified by 'TOpenDialogParams.ParentWindow' - use with 'ofNotModal' and 'ofNoClose' flag
        ofStayOnTop,            //* The dialog will stay on top of other windows
        ofNoCloseButton,        //* Not possible to close the dialog if 'ofIntegrate' is used - usefull with the 'ofNotModal' flag
        ofOnSelectFilename,     //* When user selects a file (clicks on it) the hookproc will be called and/or message will be sent to the 'ParentHandle' - usefull with the 'ofNotModal' flag
        ofCompatibilityMode,    //* Reserved
        ofCustomFileInfo,       //* Custom file info is implemented, 'TOpenDialogParams.FileInfoHookProc' must be assigned
        ofCustomThumbnail,      //* Custom thumbnail is implemented, 'TOpenDialogParams.FileThumbnailHookProc' must be assigned
        ofCustomPreViewer,      //* Custom viewer is implemented, hookproc must be assigned
        ofInitialFolderIsPIDL,  //* 'TOpenDialogParams.InitialDir' is not a pointer to a string but is a PIDL
        ofReturnPIDLs,          //* Only works with open and folder dialog, also file(s) must exist
        ofBrowseArchives        //* Only works with open dialog and with 'ofReturnPIDLs'
    );
    TOpenOptions = set of TOpenOption;

type
    TOpenOptionEx = (ofExNoPlacesBar, ofExNoStatusBar, ofExNoDrivesFavsRecents, ofExNoFileName, ofExNoFilter, ofExNoToolBar, ofExNoAddToFavs, ofExNoTreeViewAndToolBar, ofExNoPathAndFilter, ofExNoTextButtons, ofExNoOpenButtons, ofExShellOpen);
    TOpenOptionsEx = set of TOpenOptionEx;

type
    OFNHookProc = function(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT; stdcall;

type
    FDFileInfoHookProc = function(FileName: PWideChar; PIDL: PItemIDList; var FileInformation: PWideChar; User: Pointer): Bool; stdcall;
    FDFileThumbnailHookProc = function(FileName: PWideChar; PIDL: Pointer; Width: Integer; Height: Integer; var Thumbnail: HBitmap; var OriginalWidth: Int64; var OriginalHeight: Int64; User: Pointer): Bool; stdcall;

//* Filesystem dialogs types
type
    TFSDialogType = (fsdtOpenDialog, fsdtSaveDialog, fsdtFolderDialog);

type
    TOFNOTIFY = {$IFDEF WIN32}packed{$ENDIF} record
        hdr: TNMHdr;
        lpOFN: Pointer; //tagOFN;
        pszFile: PWideChar;
    end;

Const
    FD_DW_DEFAULT = $00000000;

type
    TOpenDialogAppearance = record
        UIColor: Cardinal;                              //* ARGB (use FD_DW_DEFAULT = no change)
        UIFontColor: Cardinal;                          //* ARGB
        ListerBckgColor: Cardinal;                      //* ARGB
        ListerFontColor: Cardinal;                      //* ARGB
        ListerGridLines: LongBool;
        ListerHeaderGradientTopColor: Cardinal;         //* ARGB
        ListerHeaderGradientBottomColor: Cardinal;      //* ARGB
        ListerHeaderBrightColor: Cardinal;              //* ARGB
        ListerHeaderDarkColor: Cardinal;                //* ARGB
        ListerHeaderFontColor: Cardinal;                //* ARGB
        ListerSelectionAlpha: Byte;                     //* A
        ListerSelectionGradientTopColor: Cardinal;      //* ARGB
        ListerSelectionGradientBottomColor: Cardinal;   //* ARGB
        ListerSelectionFrameColor: Cardinal;            //* ARGB
        ListerHotTrackAlpha: Byte;                      //* A
        ListerHotTrackGradientTopColor: Cardinal;       //* ARGB
        ListerHotTrackGradientBottomColor: Cardinal;    //* ARGB
        ListerHotTrackFrameColor: Cardinal;             //* ARGB
        VCLStyleTheme: PChar;                           //* For example 'AQUA GRAPHITE' or a file name of a Delphi style file 'C:\Carbon.vsf'
    end;
    POpenDialogAppearance = ^TOpenDialogAppearance;

//* Filesystem dialog Open file/Save file dialog params
type
    TOpenDialogParams = record
        DefaultExt: PChar;          //* Default extension
        FileName: PChar;            //* Default file name for Open/Save and Initial dir
        Filters: PChar;             //* Filters eg.: 'Image files (*.bmp,*.jpg)|*.bmp;*.jpg|All files|*.*'
        FilterIndex: Integer;       //* Filter to use default, 1-first, 2-second...
        HelpContext: Integer;       //* Help context (not yet implemented)
        InitialDir: PChar;          //* Initial folder, this is extracted first from FileName, if no FileName specified settings are used to decide which folder to use, if 'ofInitialFolderIsPIDL' is set this should be a pointer to ItemIDList (a PItemIDList)
        Options: TOpenOptions;      //* Dialog options (not all implemented so far), OR TOpenOption flags together
        OptionsEx: TOpenOptionsEx;  //* Extra Dialog options, specify panels to hide (not all implemented), OR TOpenOptionEx flags together
        Title: PChar;               //* Title to display as window's caption
        ParentWindow: HWND;         //* Parent window handle (needed for HookProc)
        DialogGUID: TGUID;          //* Specifically identificate the dialog (separate dialogs inside an application - you can get the same dialog always with this flag)
        HookProc: OFNHookProc;      //* Hookproc (There are two: 1. for not modal mode result, and 2. for filter change event (avoid using it) - ParentWindow handle is also needed)
        DialogType: TFSDialogType;  //* Type of the dialog: Open/Save/Folder
        //FileSize: Int64;			//* Estimated file size (will warn if won't fit in the selected directory before performing any saving)
        User: Pointer;              //* User variable
        FileInfoHookProc: FDFileInfoHookProc; //* Callback that gives back custom file info
        FileInfoColumnName: PWideChar;  //* Column name used for custom file info
        FileThumbnailHookProc: FDFileThumbnailHookProc; //* Callback that gives back thumbnail
        Appearance: POpenDialogAppearance; //* Set to nil to use default settings
        AdditionalParameters: PChar; //* String list of additional parameters
    end;
    POpenDialogParams = ^TOpenDialogParams;

//* Filesystem dialog result
type
    TOpenDialogResults = record
        Successfull: BOOL;          //* True: User pressed the Open/Save button or double-clicked on a file, False: User pressed Cancel or closed the window, or error (there is fallback to comdlg32.dll though)
        FileName: PChar;            //* The filename with full path the user selected (only one) ending with a double null: "C:\Dir\Data.txt" + #0#0
        FileNameSize: Int64;        //* FileName size in bytes (should be always at least MAX_PATH)
        FileCount: Cardinal;        //* Number of files, if user selected multiple files then FileCount > 1
        FileNames: PChar;           //* List of files in format "C:\Dir\Data.txt" + #0 + "D:\Folder\DataX.txt" separated by null characters ending with a double null
        FileNamesSize: Int64;       //* FileNames size in bytes (it's up to you to validate last entry if all selected filenames don't fit into the FileNames - but always check with FileExists())
        FilterIndex: Integer;       //* Filter's index the user selected 1-first, 2-second, etc...
    end;
    POpenDialogResults = ^TOpenDialogResults;

//* FDM_ON_SELECT_FILE_NAME in WPARAM
type
    TOpenDialogOnSelectFileName = record
        FileName: PChar;            //* The filename with full path the user selected (only one)
        Item: PItemIDList;          //* File's PIDL
    end;
    POpenDialogOnSelectFileName = ^TOpenDialogOnSelectFileName;

//* FDM_GET_CURRENT_FOLDER in WPARAM
type
    TOpenDialogOnCurrentFolder = record
        Folder: PChar;              //* The full path the user selected (only one)
        Item: PItemIDList;          //* File' PIDL
    end;
    POpenDialogOnCurrentFolder = ^TOpenDialogOnCurrentFolder;

//* FDM_SELECT_PATTERN in LPARAM
type
    TOpenDialogSelectPattern = record
        Pattern: PChar;             //* Pattern string
        CaseSensitive: LongBool;    //*
    end;
    POpenDialogSelectPattern = ^TOpenDialogSelectPattern;

type
    TFDSearchProgress = record
        FileName: String;
    end;
    PFDSearchProgress = ^TFDSearchProgress;

//* FDM_STYLE_CHANGED in WPARAM
type
    TFDStyleChanged = record
        StyleName: String;
    end;
    PFDStyleChanged = ^TFDStyleChanged;

//* Exported functions
type
    t_OpenDialog = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
    t_SaveDialog = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
    t_FolderDialog = function(Params: TOpenDialogParams; var Results: TOpenDialogResults): BOOL; stdcall;
    t_FilesystemDialogsSettings = function(Params: PChar): BOOL; stdcall;
    t_FreeDialogResults = function(var Struct: TOpenDialogResults): Boolean; stdcall;

var
    FilesystemDialogs_OpenDialog: t_OpenDialog;
    FilesystemDialogs_SaveDialog: t_SaveDialog;
    FilesystemDialogs_FolderDialog: t_FolderDialog;
    FilesystemDialogs_Settings: t_FilesystemDialogsSettings;
    FilesystemDialogs_FreeDialogResults: t_FreeDialogResults;

var
    FilesystemDialogsDLLHandle: THandle = 0;
    FilesystemDialogsDLLLoaded: Boolean = False;

    function InitFilesystemDialogsLibrary: Boolean;
    function FreeFilesystemDialogsLibrary: Boolean;

implementation

Uses
    SysUtils;

function InitFilesystemDialogsLibrary: Boolean;
begin
    FilesystemDialogsDLLHandle := SafeLoadLibrary(PChar(FILENAME_DLL_FD));
    Result := FilesystemDialogsDLLHandle <> 0;
    if Result then begin
        FilesystemDialogs_OpenDialog            := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_OpenDialog));
        FilesystemDialogs_SaveDialog            := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_SaveDialog));
        FilesystemDialogs_FolderDialog          := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FolderDialog));
        FilesystemDialogs_Settings              := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_Settings));
        FilesystemDialogs_FreeDialogResults     := GetProcAddress(FilesystemDialogsDLLHandle, PChar(NAME_FD_FreeDialogResults));
    end;
    if (@FilesystemDialogs_OpenDialog = nil)
    OR (@FilesystemDialogs_SaveDialog = nil)
    OR (@FilesystemDialogs_FolderDialog = nil)
    OR (@FilesystemDialogs_Settings = nil)
    OR (@FilesystemDialogs_FreeDialogResults = nil)
        then Result := False;
    if Result
        then FilesystemDialogsDLLLoaded := True;
end;

function FreeFilesystemDialogsLibrary: Boolean;
begin
    Result := False;
    if FilesystemDialogsDLLHandle <> 0 then begin
        Result := FreeLibrary(FilesystemDialogsDLLHandle);
        if Result
            then FilesystemDialogsDLLLoaded := False;
    end;
end;

Initialization

    InitFilesystemDialogsLibrary;

Finalization

    FreeFilesystemDialogsLibrary;

end.
