//********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs Library 2.1.3.6840  3delite 2006-2024                                                                *
//*     See Filesystem Dialogs Library ReadMe.txt for details.                                                                   *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* 	http://www.shareit.com/product.html?productid=300958160                                                                  *
//* Shareware License: 95                                                                                                       *
//* 	http://www.shareit.com/product.html?productid=300070165                                                                  *
//* Commercial License: 450                                                                                                     *
//* 	http://www.shareit.com/product.html?productid=300070166                                                                  *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                          *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit FilesystemDialogsPanel;

{$IF CompilerVersion >= 24}
    {$LEGACYIFEND ON}
{$IFEND}

interface

Uses
    SysUtils,
    Windows,
    Messages,
    ShlObj,
    Controls,
    ExtCtrls,
    Classes,
    FilesystemDialogsDynamic;

type
    TFilesystemDialogPanel = class;

    TFilesystemDialogOnCreateEvent = procedure (Sender: TFilesystemDialogPanel) of object;
    TFilesystemDialogOnSelectFileNameEvent = procedure (Sender: TFilesystemDialogPanel; FileName: String; PIDL: PItemIDList) of object;
    TFilesystemDialogOnSelectFileNamesEvent = procedure (Sender: TFilesystemDialogPanel; FileNames: TStringList; PIDL: PItemIDList) of object;
    TFilesystemDialogOnFileInfoHookProcEvent = function (Sender: TFilesystemDialogPanel; FileName: String; PIDL: PItemIDList; var FileInformation: String): Bool of object;
    TFilesystemDialogOnFileThumbnailHookProcEvent = function (Sender: TFilesystemDialogPanel; FileName: String; PIDL: PItemIDList; Width: Integer; Height: Integer; var Thumbnail: HBitmap; var OriginalWidth: Int64; var OriginalHeight: Int64): Bool of object;
    //TFilesystemDialogOnCustomPreviewerEvent = procedure (Sender: TFilesystemDialogPanel; Handle: THandle; Msg: NativeUInt) of object;

    TFDStyle = (
        fdsWindows,
        fdsAmakrits,
        fdsAmethystKamri,
        fdsAquaGraphite,
        fdsAquaLightSlate,
        fdsAuric,
        fdsCarbon,
        fdsCharcoalDarkSlate,
        fdsCobaltXEMedia,
        fdsCyanDusk,
        fdsCyanNight,
        fdsEmeraldLightSlate,
        fdsGlossy,
        fdsGoldenGraphite,
        fdsIcebergClassico,
        fdsLavenderClassico,
        fdsLight,
        fdsLuna,
        fdsMetropolisUIBlack,
        fdsMetropolisUIBlue,
        fdsMetropolisUIDark,
        fdsMetropolisUIGreen,
        fdsObsidian,
        fdsRubyGraphite,
        fdsSapphireKamri,
        fdsSilver,
        fdsSlateClassico,
        fdsSmokeyQuartzKamri,
        fdsTabletDark,
        fdsTurquoiseGray
    );

    {$IF CompilerVersion >= 23}
    [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
    {$IFEND}
    TFilesystemDialogPanel = class (TPanel)
    private
        FParentPanel: TPanel;
        FDialogType: TFSDialogType;
        FOpenOptions: TOpenOptions;
        FOpenOptionsEx: TOpenOptionsEx;
        FFileName: String;
        FInitialFolder: String;
        FFilters: String;
        FFilterIndex: Integer;
        FOnCreateEvent: TFilesystemDialogOnCreateEvent;
        FOnSelectFileNameEvent: TFilesystemDialogOnSelectFileNameEvent;
        FOnSelectFileNamesEvent: TFilesystemDialogOnSelectFileNamesEvent;
        FOnFileInfoHookProcEvent: TFilesystemDialogOnFileInfoHookProcEvent;
        FOnFileThumbnailHookProcEvent: TFilesystemDialogOnFileThumbnailHookProcEvent;
        //FOnCustomPreviewerEvent: TFilesystemDialogOnCustomPreviewerEvent;
        FDialogParams: TOpenDialogParams;
        FGUID: TGUID;
        FCustomFileInfoColumnCaption: String;
        FStyle: TFDStyle;
        FUsedStyleString: String;
        FUIDPIScaling: Cardinal;
        procedure ResizeEvent(Sender: TObject);
        procedure ShowDialog;
        function GetGuid: String;
        procedure SetGuid(const Value: String);
    public
        FilesystemDialogHandle: THandle;
        Constructor Create(AOwner: TComponent); override;
        Destructor Destroy; override;
        function GetUseStyle: PChar;
    published
        property Options: TOpenOptions read FOpenOptions write FOpenOptions;
        property OptionsEx: TOpenOptionsEx read FOpenOptionsEx write FOpenOptionsEx;
        property OnCreate: TFilesystemDialogOnCreateEvent read FOnCreateEvent write FOnCreateEvent;
        property OnSelectFileName: TFilesystemDialogOnSelectFileNameEvent read FOnSelectFileNameEvent write FOnSelectFileNameEvent;
        property OnSelectFileNames: TFilesystemDialogOnSelectFileNamesEvent read FOnSelectFileNamesEvent write FOnSelectFileNamesEvent;
        property OnFileInfoHookProc: TFilesystemDialogOnFileInfoHookProcEvent read FOnFileInfoHookProcEvent write FOnFileInfoHookProcEvent;
        property OnFileThumbnailHookProc: TFilesystemDialogOnFileThumbnailHookProcEvent read FOnFileThumbnailHookProcEvent write FOnFileThumbnailHookProcEvent;
        //property OnCustomPreviewer: TFilesystemDialogOnCustomPreviewerEvent read FOnCustomPreviewerEvent write FOnCustomPreviewerEvent;
        property DialogType: TFSDialogType read FDialogType write FDialogType default fsdtOpenDialog;
        property FileName: String read FFileName write FFileName;
        property InitialFolder: String read FInitialFolder write FInitialFolder;
        property Filters: String read FFilters write FFilters;
        property FilterIndex: Integer read FFilterIndex write FFilterIndex;
        property GUID: String read GetGUID write SetGUID;
        property CustomFileInfoColumnCaption: String read FCustomFileInfoColumnCaption write FCustomFileInfoColumnCaption;
        property Style: TFDStyle read FStyle write FStyle;
        property UIDPIScaling: Cardinal read FUIDPIScaling write FUIDPIScaling default 0;
    end;

    procedure Register;

    function TFDStyle2String(FDStyle: TFDStyle): String;

var
    FilesystemDialogsPanelGlobalStyle: String;

implementation

procedure Register;
begin
    RegisterComponents('3delite', [TFilesystemDialogPanel]);
end;

function TFDStyle2String(FDStyle: TFDStyle): String;
begin
    Result := '';
    case FDStyle of
        fdsWindows: Result := 'Windows';
        fdsAmakrits: Result := 'Amakrits';
        fdsAmethystKamri: Result := 'Amethyst Kamri';
        fdsAquaGraphite: Result := 'Aqua Graphite';
        fdsAquaLightSlate: Result := 'Aqua Light Slate';
        fdsAuric: Result := 'Auric';
        fdsCarbon: Result := 'Carbon';
        fdsCharcoalDarkSlate: Result := 'Charcoal Dark Slate';
        fdsCobaltXEMedia: Result := 'Cobalt XEMedia';
        fdsCyanDusk: Result := 'Cyan Dusk';
        fdsCyanNight: Result := 'Cyan Night';
        fdsEmeraldLightSlate: Result := 'Emerald Light Slate';
        fdsGlossy: Result := 'Glossy';
        fdsGoldenGraphite: Result := 'Golden Graphite';
        fdsIcebergClassico: Result := 'Iceberg Classico';
        fdsLavenderClassico: Result := 'Lavender Classico';
        fdsLight: Result := 'Light';
        fdsLuna: Result := 'Luna';
        fdsMetropolisUIBlack: Result := 'Metropolis UI Black';
        fdsMetropolisUIBlue: Result := 'Metropolis UI Blue';
        fdsMetropolisUIDark: Result := 'Metropolis UI Dark';
        fdsMetropolisUIGreen: Result := 'Metropolis UI Green';
        fdsObsidian: Result := 'Obsidian';
        fdsRubyGraphite: Result := 'Ruby Graphite';
        fdsSapphireKamri: Result := 'Sapphire Kamri';
        fdsSilver: Result := 'Silver';
        fdsSlateClassico: Result := 'Slate Classico';
        fdsSmokeyQuartzKamri: Result := 'Smokey Quartz Kamri';
        fdsTabletDark: Result := 'TabletDark';
        fdsTurquoiseGray: Result := 'Turquoise Gray';
    end;
end;

function OFNHookProc(Wnd: HWND; Msg: UINT; W_Param: WPARAM; L_Param: LPARAM): UINT stdcall;
var
    Results: POpenDialogResults;
    FileName: String;
    Struct: Pointer;
    PFileName: PChar;
    FilesystemDialogPanel: TFilesystemDialogPanel;
    FileNames: TStringList;
begin
    //* Results arrived
    if Msg = FDM_RESULTS then begin
        Results := POpenDialogResults(W_Param);
        FilesystemDialogPanel := TFilesystemDialogPanel(POpenDialogParams(Pointer(L_Param)).User);
        //* Extract files (multiple) ending with a double null
        FileNames := TStringList.Create;
        try
            PFileName := Results.FileNames;
            while PFileName^ <> #0 do begin
                FileName := PFileName;
                FileNames.Append(FileName);
                Inc(PFileName, Length(PFileName) + 1);
            end;
            if FileNames.Count > 0 then begin
                if Assigned(FilesystemDialogPanel.OnSelectFileNames) then begin
                    FilesystemDialogPanel.OnSelectFileNames(FilesystemDialogPanel, FileNames, nil);
                end;
            end;
        finally
            FreeAndNil(FileNames);
        end;
    end;
    //* User selected a file
    if Msg = FDM_ON_SELECT_FILE_NAME then begin
        FilesystemDialogPanel := TFilesystemDialogPanel(POpenDialogParams(Pointer(L_Param)).User);
        if Assigned(FilesystemDialogPanel.OnSelectFileName) then begin
            FilesystemDialogPanel.OnSelectFileName(FilesystemDialogPanel, POpenDialogOnSelectFileName(W_Param).FileName, POpenDialogOnSelectFileName(W_Param).Item);
        end;
    end;
    //* Free the memory as it is not needed anymore
    if Msg = FDM_FREEMEM then begin
        Struct := Pointer(W_Param);
        //* Free our appearance record if we have one
        if Assigned(POpenDialogParams(L_PARAM).Appearance) then begin
            Dispose(POpenDialogParams(L_PARAM).Appearance);
        end;
        //* Free memory
        FilesystemDialogs_FreeDialogResults(TOpenDialogResults(Struct^));
    end;
    //* The window handle arrived
    if Msg = FDM_WINDOW_HANDLE then begin
        FilesystemDialogPanel := TFilesystemDialogPanel(POpenDialogParams(Pointer(L_Param)).User);
        FilesystemDialogPanel.FilesystemDialogHandle := W_Param;
    end;
    Result := 1;
end;

function FileInfoHookProc(FileName: PWideChar; PIDL: PItemIDList; var FileInformation: PWideChar; User: Pointer): Bool; stdcall;
var
    FilesystemDialogPanel: TFilesystemDialogPanel;
    Information: String;
begin
    Result := False;
    FilesystemDialogPanel := TFilesystemDialogPanel(User);
    if Assigned(FilesystemDialogPanel.OnFileInfoHookProc) then begin
        Result := FilesystemDialogPanel.OnFileInfoHookProc(FilesystemDialogPanel, FileName, PIDL, Information);
        if Result then begin
            StrLCopy(FileInformation, PChar(Information), 254);
        end;
    end;
end;

function FileThumbnailHookProc(FileName: PWideChar; PIDL: Pointer; Width: Integer; Height: Integer; var Thumbnail: HBitmap; var OriginalWidth: Int64; var OriginalHeight: Int64; User: Pointer): Bool; stdcall;
var
    FilesystemDialogPanel: TFilesystemDialogPanel;
begin
    Result := False;
    FilesystemDialogPanel := TFilesystemDialogPanel(User);
    if Assigned(FilesystemDialogPanel.OnFileThumbnailHookProc) then begin
        Result := FilesystemDialogPanel.OnFileThumbnailHookProc(FilesystemDialogPanel, FileName, PIDL, Width, Height, Thumbnail, OriginalWidth, OriginalHeight);
    end;
end;

{ TFilesystemDialogsPanel }

constructor TFilesystemDialogPanel.Create(AOwner: TComponent);
begin
    inherited;
    BevelOuter := bvLowered;
    FDialogType := fsdtOpenDialog;
    FParentPanel := TPanel.Create(Self);
    FParentPanel.Parent := Self;
    FParentPanel.Align := alClient;
    FParentPanel.BevelOuter := bvNone;
    FParentPanel.Caption := '';
    FParentPanel.OnResize := ResizeEvent;
    if ComponentState = [csDesigning, csFreeNotification] then begin
        CreateGUID(FGUID);
    end;
    //* Clear
    ZeroMemory(@FDialogParams, SizeOf(FDialogParams));
    if Assigned(FOnCreateEvent) then begin
        FOnCreateEvent(Self);
    end;
    if NOT (csDesigning in ComponentState) then begin
        ShowDialog;
    end;
end;

destructor TFilesystemDialogPanel.Destroy;
begin
    if FilesystemDialogHandle <> 0 then begin
        SendMessage(FilesystemDialogHandle, WM_CLOSE, 0, 0);
    end;
    FreeAndNil(FParentPanel);
    inherited;
end;

procedure TFilesystemDialogPanel.ResizeEvent(Sender: TObject);
begin
    if FilesystemDialogHandle <> 0 then begin
        SendMessage(FilesystemDialogHandle, FDM_REALIGN_INTEGRATED_WINDOW, 0, 0);
    end;
end;

procedure TFilesystemDialogPanel.ShowDialog;
var
    Results: TOpenDialogResults;
begin
    //* Set params
    FDialogParams.FileName := PChar(FileName);
    FDialogParams.InitialDir := PChar(InitialFolder);
    FDialogParams.Filters := PChar(Filters);
    FDialogParams.FilterIndex := FilterIndex;
    FDialogParams.FileInfoColumnName := PChar(FCustomFileInfoColumnCaption);
    FDialogParams.Options := FOpenOptions;
    FDialogParams.OptionsEx := FOpenOptionsEx;
    FDialogParams.ParentWindow := FParentPanel.Handle;
    FDialogParams.HookProc := OFNHookProc;
    //* Hook procs?
    if ofCustomFileInfo in FDialogParams.Options then begin
        FDialogParams.FileInfoHookProc := FileInfoHookProc;
    end;
    if ofCustomThumbnail in FDialogParams.Options then begin
        FDialogParams.FileThumbnailHookProc := FileThumbnailHookProc;
    end;
    //* Hard coded needed flags
    FDialogParams.Options := FDialogParams.Options + [ofNotModal];
    FDialogParams.Options := FDialogParams.Options + [ofNoClose];
    FDialogParams.Options := FDialogParams.Options + [ofIntegrate];
    FDialogParams.Options := FDialogParams.Options + [ofOnSelectFileName];
    FDialogParams.Options := FDialogParams.Options + [ofNoCloseButton];
    //* Style
    if FilesystemDialogsPanelGlobalStyle <> '' then begin
        New(FDialogParams.Appearance);
        FDialogParams.Appearance.VCLStyleTheme := PChar(FilesystemDialogsPanelGlobalStyle);
    end else begin
        if FStyle <> fdsWindows then begin
            New(FDialogParams.Appearance);
            FDialogParams.Appearance.VCLStyleTheme := GetUseStyle;
        end;
    end;
    //* DPI
    if FUIDPIScaling <> 0 then begin
        FDialogParams.AdditionalParameters := PChar('UIDPIScaling=' + IntToStr(FUIDPIScaling));
    end;
    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    FDialogParams.DialogGUID := FGUID;
    //* ID our dialog panel instance
    FDialogParams.User := Pointer(Self);
    //* Set up result structure
    ZeroMemory(@Results, SizeOf(Results));
    //* Show the dialog
    case FDialogType of
        fsdtOpenDialog: FilesystemDialogs_OpenDialog(FDialogParams, Results);
        fsdtSaveDialog: FilesystemDialogs_SaveDialog(FDialogParams, Results);
        fsdtFolderDialog: FilesystemDialogs_FolderDialog(FDialogParams, Results);
    end;
end;

function TFilesystemDialogPanel.GetGUID: String;
begin
    Result := GUIDToString(FGUID);
end;

procedure TFilesystemDialogPanel.SetGUID(const Value: String);
begin
    FGUID := StringToGUID(Value);
end;

function TFilesystemDialogPanel.GetUseStyle: PChar;
begin
    FUsedStyleString := TFDStyle2String(FStyle);
    Result := PChar(FUsedStyleString);
end;

end.
