unit LogicalProcessorInformation;

interface

type
    TLogicalProcessorInformation = record
        LogicalProcessorCount: Integer;
        NumaNodeCount: Integer;
        ProcessorCoreCount: Integer;
        ProcessorL1CacheCount: Integer;
        ProcessorL2CacheCount: Integer;
        ProcessorL3CacheCount: Integer;
        ProcessorPackageCount: Integer;
    end;

    function GetLogicalProcessorInfo: TLogicalProcessorInformation;

var
    ProcessorInformation: TLogicalProcessorInformation;

implementation

Uses
    System.SysUtils,
    Windows;

function CountSetBits(BitMask: NativeUInt): Integer;
var
    lShift, i: Integer;
    BitTest: NativeUInt;
begin
    lShift := SizeOf(NativeUInt) * 8 - 1;
    Result := 0;
    BitTest := 1 shl lShift;
    for i := 0 to lShift do begin
        if (BitMask and BitTest) <> 0 then begin
            Inc(Result);
        end;
        BitTest := BitTest shr 1;
    end;
end;

function GetLogicalProcessorInfo: TLogicalProcessorInformation;
var
    i: Integer;
    ReturnLength: DWORD;
    InfoBuffer: array of TSystemLogicalProcessorInformation;
begin
    Result.LogicalProcessorCount := 0;
    Result.NumaNodeCount := 0;
    Result.ProcessorCoreCount := 0;
    Result.ProcessorL1CacheCount := 0;
    Result.ProcessorL2CacheCount := 0;
    Result.ProcessorL3CacheCount := 0;
    Result.ProcessorPackageCount := 0;
    SetLength(InfoBuffer, 256);
    ReturnLength := Length(InfoBuffer) * SizeOf(TSystemLogicalProcessorInformation);
    if NOT GetLogicalProcessorInformation(@InfoBuffer[0], ReturnLength) then begin
        if GetLastError = ERROR_INSUFFICIENT_BUFFER then begin
            SetLength(InfoBuffer, ReturnLength div SizeOf(TSystemLogicalProcessorInformation) + 1);
        if NOT GetLogicalProcessorInformation(@InfoBuffer[0], ReturnLength) then begin
            RaiseLastOSError;
        end;
    end else
        RaiseLastOSError;
    end;
    SetLength(InfoBuffer, ReturnLength div SizeOf(TSystemLogicalProcessorInformation));
    for i := 0 to High(InfoBuffer) do begin
        case InfoBuffer[i].Relationship of
            RelationNumaNode: Inc(Result.NumaNodeCount);
            RelationProcessorCore: begin
                Inc(Result.ProcessorCoreCount);
                Result.LogicalProcessorCount := Result.LogicalProcessorCount + CountSetBits(InfoBuffer[i].ProcessorMask);
            end;
            RelationCache: begin
                if (InfoBuffer[i].Cache.Level = 1) then begin
                    Inc(Result.ProcessorL1CacheCount);
                end else if (InfoBuffer[i].Cache.Level = 2) then begin
                    Inc(Result.ProcessorL2CacheCount);
                end else if (InfoBuffer[i].Cache.Level = 3) then begin
                    Inc(Result.ProcessorL3CacheCount);
                end;
            end;
            RelationProcessorPackage: Inc(Result.ProcessorPackageCount);
            else begin
                raise Exception.Create('Error: Unsupported LOGICAL_PROCESSOR_RELATIONSHIP value.');
            end;
        end;
    end;
end;

Initialization

    ProcessorInformation := GetLogicalProcessorInfo;

end.
