//********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs Library 2.1.3  3delite 2006-2023                                                                     *
//*     See Filesystem Dialogs Library Readme.txt for details.                                                                   *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* 	http://www.shareit.com/product.html?productid=300958160                                                                  *
//* Shareware License: 95                                                                                                       *
//* 	http://www.shareit.com/product.html?productid=300070165                                                                  *
//* Commercial License: 450                                                                                                     *
//* 	http://www.shareit.com/product.html?productid=300070166                                                                  *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                          *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit Unit1;

interface

uses
  Windows, Messages, UITypes, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, FilesystemDialogsDynamic, ExtCtrls, StdCtrls, Vcl.AppEvnts;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Edit3: TEdit;
    Label1: TLabel;
    Edit4: TEdit;
    Label3: TLabel;
    Button8: TButton;
    Button12: TButton;
    Button7: TButton;
    Button1: TButton;
    Memo1: TMemo;
    PanelParent: TPanel;
    Edit1: TEdit;
    GroupBox2: TGroupBox;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    CheckBox6: TCheckBox;
    CheckBox7: TCheckBox;
    CheckBox8: TCheckBox;
    GroupBox3: TGroupBox;
    CheckBoxStyle: TCheckBox;
    ComboBoxStyle: TComboBox;
    ApplicationEvents1: TApplicationEvents;
    GroupBox4: TGroupBox;
    CheckBoxMultipleFiles: TCheckBox;
    CheckBoxOfNotModal: TCheckBox;
    CheckBoxOfNoClose: TCheckBox;
    CheckBoxOfIntegrate: TCheckBox;
    CheckBoxOnSelectFile: TCheckBox;
    CheckBoxCustomFileInfo: TCheckBox;
    CheckBoxNoTextButtons: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure Button12Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PanelParentResize(Sender: TObject);
    procedure ApplicationEvents1Message(var Msg: tagMSG; var Handled: Boolean);
  private
    { Private declarations }
  protected

  public
    { Public declarations }
  end;

var
    Form1: TForm1;
    //* The variables and handles are global so we can make use of the ofNotModal/ofIntegrate mode
    //* For modal mode thay can be inside the procedure/function you are calling from
    FDWinHandle: THandle = 0;
    Params: TOpenDialogParams;
    Results: TOpenDialogResults;
    NewScale: Integer;

implementation

{$R *.dfm}

Uses
    ShlObj,
    Winapi.KnownFolders,
    DPIFunctions;

function OFNHookProc(Wnd: HWND; Msg: UINT; W_Param: WPARAM; L_Param: LPARAM): UINT stdcall;
//* 'L_Param' always contains the pointer to the parameters used when invoking the dialog (POpenDialogParams)
var
    Res: POpenDialogResults;
    Text: String;
    Struct: Pointer;
begin
    //* Sent results
    if Msg = FDM_RESULTS then begin
        Res := POpenDialogResults(W_Param);
        //* Extract first! filename (see the Button1Click procedure on how to extract multiple files)
        Text := PChar(Res.FileName);
        Form1.Memo1.Lines.Add(Text);
    end;
    //* User selected a file
    if Msg = FDM_ON_SELECT_FILE_NAME then begin
        Text := PChar(POpenDialogOnSelectFileName(W_Param).FileName);
        Form1.Memo1.Lines.Add(Text);
    end;
    //* Free the memory as it is not needed anymore
    if Msg = FDM_FREEMEM then begin
        //* Free our appearance record if we have one
        if Assigned(POpenDialogParams(L_PARAM).Appearance) then begin
            Dispose(POpenDialogParams(L_PARAM).Appearance);
        end;
        //* Free results record
        Struct := Pointer(W_Param);
        FilesystemDialogs_FreeDialogResults(TOpenDialogResults(Struct^));
    end;
    //* The window handle is sent back
    if Msg = FDM_WINDOW_HANDLE then begin
        FDWinHandle := W_Param;
    end;
    Result := 0;
end;

function FileInfoHookProc(FileName: PWideChar; Item: PItemIDList; var FileInformation: PWideChar; User: Pointer): Bool; stdcall;
var
    Info: String;
begin
    //* As custom information we display the file name in upper case
    //* Custom file info is limited to 255 chars
    Info := ExtractFileName(FileName);
    Info := WideUpperCase(Info);
    StrLCopy(FileInformation, PWideChar(Info), 255);
    Result := True;
end;

function GetFileNameFromPIDL(Item: PItemIDList): String;
var
    FBuf: array[0..MAX_PATH] of Char;
begin
    SHGetPathFromIDList(Item, @FBuf[0]);
    Result := FBuf;
end;

procedure TForm1.ApplicationEvents1Message(var Msg: tagMSG; var Handled: Boolean);
var
    FocusedHandle: HWND;
begin
    //* Workaround for ofIntegrate mode (focus returns to parent window with cursor keys, so re-set focused control in case)
    if Msg.message = WM_KEYDOWN then begin
        if (Msg.wParam = VK_UP)
        OR (Msg.wParam = VK_DOWN)
        OR (Msg.wParam = VK_LEFT)
        OR (Msg.wParam = VK_RIGHT)
        then begin
            FocusedHandle := GetFocus;
            SendMessage(FocusedHandle, WM_KEYDOWN, Msg.wParam, Msg.lParam);
            Windows.SetFocus(FocusedHandle);
            Handled := True;
        end;
     end;
end;

procedure TForm1.Button12Click(Sender: TObject);
var
    Parameters: String;
    Success: Boolean;
begin
    //* Scale user interface DPI by a custom amount (it not specified, Windows' setting is used)
    //* 96 = 100%, 120 = 125%, 144 = 150%, etc.
    Parameters := 'UIDPIScaling=' + IntToStr(NewScale);
    //* Call FilesystemDialogs_SettingsWindow()
    Success := FilesystemDialogs_Settings(PChar(Parameters));
    //* Use result
    Memo1.Clear;
    if Success
        then Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): OK.')
        else Memo1.Lines.Append('FilesystemDialogs_SettingsWindow(): ERROR.');
end;

procedure TForm1.Button1Click(Sender: TObject);
var
    StartupFolder: PItemIDList;
    PFileName: PChar;
    //PPIDL: PByte;
    //i: Integer;
    //FileName: String;
begin
    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.DialogType := fsdtOpenDialog;
    Params.FileName := PChar(Edit1.Text);
    //* Get PIDL for initial folder, if 'Params.FileName' is set path is extracted from it
    StartupFolder := nil;
    SHGetKnownFolderIDList(FOLDERID_ComputerFolder, 0, 0, StartupFolder);
    Params.InitialDir := PChar(StartupFolder);
    //* If 'ofInitialFolderIsPIDL' is not set 'Params.InitialDir' is considered as a PWideChar folder name
    Params.Options := Params.Options + [ofInitialFolderIsPIDL];
    //* Filter (file types) list
    Params.Filters := PChar(Edit3.Text);
    Params.FilterIndex := StrToInt(Edit4.Text);
    //* Multi or single
    if CheckBoxMultipleFiles.Checked
        then Params.Options := Params.Options + [ofAllowMultiSelect]
        else Params.Options := Params.Options - [ofAllowMultiSelect];
    //* Modal or not (the NOT modal form is the suggested methode)
    //* This requires a Handle which to the message will be sent and/or a HookProc
    //* This is needed to free the resources at the session end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := PanelParent.Handle;
        //fHookProc := OFNHookProc;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofNotModal];
    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofNoClose];
    //* Integrate the dialog into a parent panel
    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofIntegrate];
    //* Send a message when selection changes
    //* This requires a Handle which to the message will be sent and/or a HookProc
    if CheckBoxOnSelectFile.Checked then begin
        Params.Options := Params.Options + [ofOnSelectFileName];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofOnSelectFileName];
    //* Display custom file info
    if CheckBoxCustomFileInfo.Checked then begin
        Params.Options := Params.Options + [ofCustomFileInfo];
        Params.FileInfoColumnName := PWideChar('Custom information');
        Params.FileInfoHookProc := FileInfoHookProc;
    end else Params.Options := Params.Options - [ofCustomFileInfo];
    //* Hide options
    if CheckBoxNoTextButtons.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoTextButtons];
    end;
    if CheckBox1.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoStatusBar];
    end;
    if CheckBox2.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoDrivesFavsRecents];
    end;
    if CheckBox3.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoFileName];
    end;
    if CheckBox4.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoFilter];
    end;
    if CheckBox5.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoToolBar];
    end;
    if CheckBox6.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoAddToFavs];
    end;
    if CheckBox7.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoTreeViewAndToolBar];
    end;
    if CheckBox8.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoPathAndFilter];
    end;
    //* Example code to apply a black theme
    {
    //* Create struct (is freed in the HookProc)
    New(Params.Appearance);
    //* Colors are in ARGB format (alpha value usually only used for the animated list view)
    Params.Appearance.UIColor := $00000000;
    Params.Appearance.UIFontColor := $FFFFFFFF;
    Params.Appearance.ListerBckgColor := $00000000;
    Params.Appearance.ListerFontColor := $FFFFFFFF;
    Params.Appearance.ListerGridLines := False;
    Params.Appearance.ListerHeaderGradientTopColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHeaderGradientBottomColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHeaderBrightColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHeaderDarkColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHeaderFontColor := FD_DW_DEFAULT;
    Params.Appearance.ListerSelectionGradientTopColor := FD_DW_DEFAULT;
    Params.Appearance.ListerSelectionGradientBottomColor := FD_DW_DEFAULT;
    Params.Appearance.ListerSelectionFrameColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHotTrackGradientTopColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHotTrackGradientBottomColor := FD_DW_DEFAULT;
    Params.Appearance.ListerHotTrackFrameColor := FD_DW_DEFAULT;
    }
    //* Use Delpi VCL styles
    if CheckBoxStyle.Checked then begin
        //* Create struct (is freed in the HookProc)
        New(Params.Appearance);
        //* For example 'AQUA GRAPHITE' or a file name of a Delphi style file 'C:\Carbon.vsf'
        Params.Appearance.VCLStyleTheme := PChar(ComboBoxStyle.Items[ComboBoxStyle.ItemIndex]);
    end;
    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9C-F2ED-4BF7-B90E-0831BABA3093}');
    //* Return PIDL(s)
    //Params.Options := Params.Options + [ofReturnPIDLs];
    //* Scale user interface DPI by a custom amount (it not specified, Windows' setting is used)
    //* 96 = 100%, 120 = 125%, 144 = 150%, etc.
    if NewScale <> 96 then begin
        Params.AdditionalParameters := PChar('UIDPIScaling=' + IntToStr(NewScale));
    end;
    //* Set up result structure
    ZeroMemory(@Results, SizeOf(Results));
    //* Call OpenDialog()
    FilesystemDialogs_OpenDialog(Params, Results);
    //* Use result
    if Results.Successfull then begin
        //* Single file
        Edit1.Text := Results.FileName;
        //* Extract files (multiple) ending with a double null
        Memo1.Clear;
        PFileName := Results.FileNames;
        while PFileName^ <> #0 do begin
            Memo1.Lines.Append(PFileName);
            // Point to the next filename
            Inc(PFileName, Length(PFileName) + 1);
        end;
        (*
        //* If 'ofReturnPIDLs' flag is used PIDL(s) are returned. Use this code in hookproc too if needed.
        //* Single file PIDL
        FileName := GetFileNameFromPIDL(PItemIDList(Results.FileName));
        Edit1.Text := FileName;
        //* Multi-files PIDL(s)
        Memo1.Clear;
        PPIDL := PByte(Results.FileNames);
        for i := 0 to Results.FileCount - 1 do begin
            FileName := GetFileNameFromPIDL(PItemIDList(PPIDL));
            Memo1.Lines.Append(FileName);
            Inc(PPIDL, ILGetSize(PItemIDList(PPIDL)));
        end;
        *)
        //* Filter selected
        Edit4.Text := IntToStr(Params.FilterIndex);
    end;
    //* Free memory if window is not modal thus is closed (we don't need it anymore)
    //* Else it will be freed in the callback!
    if NOT CheckBoxOfNotModal.Checked then begin
        //* Free our appearance record if we have one
        if Assigned(Params.Appearance) then begin
            Dispose(Params.Appearance);
        end;
        //* Free the results
        FilesystemDialogs_FreeDialogResults(Results);
    end;
    if Assigned(StartupFolder) then begin
        ILFree(StartupFolder);
    end;
end;

procedure TForm1.Button7Click(Sender: TObject);
begin
    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.DialogType := fsdtSaveDialog;
    Params.FileName := PChar(Edit1.Text);
    Params.Filters := PChar(Edit3.Text);
    Params.FilterIndex := StrToInt(Edit4.Text);
    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9D-F2ED-4BF7-B90E-0831BABA3093}');
    //* Modal or not (the NOT modal form is the suggested methode)
    //* This requires a Handle which to the message will be sent and a HookProc that will be invoked at the end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofNotModal];
    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofNoClose];
    //* Integrate the dialog into a parent panel
    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofIntegrate];
    //* Send a message when selection changes
    //* This requires a Handle which to the message will be sent and/or a HookProc
    if CheckBoxOnSelectFile.Checked then begin
        Params.Options := Params.Options + [ofOnSelectFileName];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofOnSelectFileName];
    //* Display custom file info
    if CheckBoxCustomFileInfo.Checked then begin
        Params.Options := Params.Options + [ofCustomFileInfo];
        Params.FileInfoColumnName := PWideChar('Custom information');
        Params.FileInfoHookProc := FileInfoHookProc;
    end else Params.Options := Params.Options - [ofCustomFileInfo];
    //* Hide options
    if CheckBoxNoTextButtons.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoTextButtons];
    end;
    if CheckBox1.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoStatusBar];
    end;
    if CheckBox2.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoDrivesFavsRecents];
    end;
    if CheckBox3.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoFileName];
    end;
    if CheckBox4.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoFilter];
    end;
    if CheckBox5.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoToolBar];
    end;
    if CheckBox6.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoAddToFavs];
    end;
    if CheckBox7.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoTreeViewAndToolBar];
    end;
    if CheckBox8.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoPathAndFilter];
    end;
    //* Use Delpi VCL styles
    if CheckBoxStyle.Checked then begin
        //* Create struct (is freed in the HookProc)
        New(Params.Appearance);
        //* For example 'AQUA GRAPHITE' or a file name of a Delphi style file 'C:\Carbon.vsf'
        Params.Appearance.VCLStyleTheme := PChar(ComboBoxStyle.Items[ComboBoxStyle.ItemIndex]);
    end;
    //* Scale user interface DPI by a custom amount
    if NewScale <> 96 then begin
        Params.AdditionalParameters := PChar('UIDPIScaling=' + IntToStr(NewScale));
    end;
    //* Set up result structure
    ZeroMemory(@Results, SizeOf(Results));
    //* Call SaveDialog
    FilesystemDialogs_SaveDialog(Params, Results);
    //* Use result
    if Results.Successfull then begin
        //* Single file
        Edit1.Text := Results.FileName;
        Memo1.Clear;
        Memo1.Lines.Append(Results.FileNames);
        //* Filter selected
        Edit4.Text := IntToStr(Params.FilterIndex);
    end;
    //* Free memory if window is not modal thus is closed (we don't need it anymore)
    //* Else it will be freed in the callback!
    if NOT CheckBoxOfNoClose.Checked then begin
        FilesystemDialogs_FreeDialogResults(Results);
    end;
end;

procedure TForm1.Button8Click(Sender: TObject);
begin
    //* Set params
    ZeroMemory(@Params, SizeOf(Params));
    Params.InitialDir := PChar(ExtractFilePath(Edit1.Text));
    //* Identify this dialog (SHIFT+CTRL+G creates a new GUID)
    Params.DialogGUID := StringToGUID('{54F52A9D-F2ED-4BF7-B90E-0831BABA3093}');
    ZeroMemory(@Results, SizeOf(Results));
    //* Inititalize the result structure (memory allocation)
    //FilesystemDialogs_InitDialogResults(Results);
    //* Modal or not (the NOT modal form is the suggested methode)
    //* This requires a Handle which to the message will be sent and/or a HookProc
    //* This is needed to free the resources at the session end
    if CheckBoxOfNotModal.Checked then begin
        Params.Options := Params.Options + [ofNotModal];
        Params.ParentWindow := Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofNotModal];
    //* Browser mode
    if CheckBoxOfNoClose.Checked then begin
        Params.Options := Params.Options + [ofNoClose];
        Params.ParentWindow := Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofNoClose];
    //* Integrate the dialog into a parent panel
    if CheckBoxOfIntegrate.Checked then begin
        Params.Options := Params.Options + [ofIntegrate];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofIntegrate];
    //* Send a message when selection changes
    //* This requires a Handle which to the message will be sent and/or a HookProc
    if CheckBoxOnSelectFile.Checked then begin
        Params.Options := Params.Options + [ofOnSelectFileName];
        Params.ParentWindow := PanelParent.Handle;
        Params.HookProc := OFNHookProc;
    end else Params.Options := Params.Options - [ofOnSelectFileName];
    //* Hide panels
    if CheckBox2.Checked then begin
        Params.OptionsEx := Params.OptionsEx + [ofExNoDrivesFavsRecents];
    end;
    //* Use Delpi VCL styles
    if CheckBoxStyle.Checked then begin
        //* Create struct (is freed in the HookProc)
        New(Params.Appearance);
        //* For example 'AQUA GRAPHITE' or a file name of a Delphi style file 'C:\Carbon.vsf'
        Params.Appearance.VCLStyleTheme := PChar(ComboBoxStyle.Items[ComboBoxStyle.ItemIndex]);
    end;
    //* Scale user interface DPI by a custom amount
    if NewScale <> 96 then begin
        Params.AdditionalParameters := PChar('UIDPIScaling=' + IntToStr(NewScale));
    end;
    //* Call FolderDialog
    FilesystemDialogs_FolderDialog(Params, Results);
    //* Use result
    if Results.Successfull then begin
        //* Single file
        Edit1.Text := Results.FileName;
        Memo1.Clear;
        Memo1.Lines.Append(Results.FileName);
    end;
    //* Free memory if window is not modal thus is closed (we don't need it anymore)
    //* Else it will be freed in the callback!
    if NOT CheckBoxOfNotModal.Checked then begin
        FilesystemDialogs_FreeDialogResults(Results);
    end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    //* Close the dialog if integrated
    if FDWinHandle <> 0 then begin
        SendMessage(FDWinHandle, WM_CLOSE, 0, 0);
    end;
    //* Important to close the hosting window too
    Application.ProcessMessages;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    //* Check if FilesystemDialogs.dll is successfuly loaded
    if NOT FilesystemDialogsDLLLoaded then begin
        MessageDlg('Error loading ' + FILENAME_DLL_FD + '!', mtError, [mbOk], 0);
        Application.Terminate;
    end;
    //* Get DPI (current monitor)
    NewScale := GetSystemDPI(Self.Monitor.Handle);
    //* Apply DPI if needed to this window
    if NewScale <> 96 then begin
        Self.ScaleBy(NewScale, 96);
    end;
end;

procedure TForm1.PanelParentResize(Sender: TObject);
begin
    //* Realign the dialog if integrated
    if FDWinHandle <> 0 then begin
        SendMessage(FDWinHandle, FDM_REALIGN_INTEGRATED_WINDOW, 0, 0);
    end;
end;

end.
